/*
 *
 * file.c        File access
 *
 * Copyright (c) 1996-1998  Hubert Mantel, S.u.S.E. GmbH  (mantel@suse.de)
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/mount.h>
#include <linux/fs.h>
#include <arpa/inet.h>

#include "global.h"
#include "text.h"
#include "util.h"
#include "module.h"
#include "modparms.h"
#include "window.h"
#include "dialog.h"
#include "net.h"

static const char  *file_infofile_tm         = "/etc/install.inf";

static const char  *file_txt_language_tm       = "Language:";
static const char  *file_txt_sourcemount_tm    = "Sourcemounted:";
static const char  *file_txt_display_tm        = "Display:";
static const char  *file_txt_keymap_tm         = "Keytable:";
static const char  *file_txt_cdrom_tm          = "Cdrom:";
static const char  *file_txt_scsi_tm           = "scsi_hostadapter";
static const char  *file_txt_pcmcia_tm         = "PCMCIA:";
static const char  *file_txt_bootmode_tm       = "Bootmode:";
static const char  *file_txt_bootfloppy_tm     = "Floppy";
static const char  *file_txt_bootcd_tm         = "CD";
static const char  *file_txt_bootnet_tm        = "Net";
static const char  *file_txt_bootharddisk_tm   = "Harddisk";
static const char  *file_txt_bootftp_tm        = "FTP";
static const char  *file_txt_partition_tm      = "Partition:";
static const char  *file_txt_fstyp_tm          = "Fstyp:";
static const char  *file_txt_serverdir_tm      = "Serverdir:";
static const char  *file_txt_ip_tm             = "IP:";
static const char  *file_txt_pliphost_tm       = "PLIP-Host:";
static const char  *file_txt_netmask_tm        = "Netmask:";
static const char  *file_txt_gateway_tm        = "Gateway:";
static const char  *file_txt_server_tm         = "Server:";
static const char  *file_txt_dnsserver_tm      = "Nameserver:";
static const char  *file_txt_netdevice_tm      = "Netdevice:";
static const char  *file_txt_machine_name_tm   = "Machinename:";
static const char  *file_txt_domain_name_tm    = "Domain:";
static const char  *file_txt_bootp_wait_tm     = "Bootpwait:";
static const char  *file_txt_ftp_user_tm       = "FTP-User:";
static const char  *file_txt_ftp_passwd_tm     = "FTP-Password:";
static const char  *file_txt_ftp_proxy_tm      = "FTP-Proxy:";
static const char  *file_txt_ftp_proxy_port_tm = "FTP-Proxyport:";

static void file_get_value   (char *input_tv, char *value_tr);
static void file_trim_buffer (char *buffer_tr);

void file_write_yast_info (void)
    {
    FILE  *file_pri;
    char   line_ti [200];


    file_pri = fopen (file_infofile_tm, "w");

    strcpy (line_ti, file_txt_language_tm);
    if (language_ig == LANG_ENGLISH)
        strcat (line_ti, " english\n");
    else if (language_ig == LANG_ITALIAN)
        strcat (line_ti, " italian\n");
    else if (language_ig == LANG_FRENCH)
        strcat (line_ti, " french\n");
    else
        strcat (line_ti, " german\n");
    fprintf (file_pri, line_ti);

    strcpy (line_ti, file_txt_sourcemount_tm);
    if (!ramdisk_ig)
        strcat (line_ti, " 1\n");
    else
        strcat (line_ti, " 0\n");
    fprintf (file_pri, line_ti);

    strcpy (line_ti, file_txt_display_tm);
    if (colors_prg->has_colors)
        strcat (line_ti, " Color\n");
    else
        strcat (line_ti, " Mono\n");
    fprintf (file_pri, line_ti);

    if (keymap_tg [0])
        fprintf (file_pri, "%s %s\n", file_txt_keymap_tm, keymap_tg);
    if (cdrom_tg [0])
        fprintf (file_pri, "%s %s\n", file_txt_cdrom_tm, cdrom_tg);
    if (scsi_tg [0])
        fprintf (file_pri, "alias %s %s\n", file_txt_scsi_tm, scsi_tg);
    if (net_tg [0])
        fprintf (file_pri, "alias %s %s\n", netdevice_tg, net_tg);
    if (ppcd_tg [0])
        {
        fprintf (file_pri, "post-install paride insmod %s\n", ppcd_tg);
        fprintf (file_pri, "pre-remove paride rmmod %s\n", ppcd_tg);
        }

    if (pcmcia_chip_ig == 1 || pcmcia_chip_ig == 2)
        {
        strcpy (line_ti, file_txt_pcmcia_tm);
        if (pcmcia_chip_ig == 1)
            strcat (line_ti, " tcic\n");
        else
            strcat (line_ti, " i82365\n");
        fprintf (file_pri, line_ti);
        }

    strcpy (line_ti, file_txt_bootmode_tm);
    strcat (line_ti, " ");
    switch (bootmode_ig)
        {
        case BOOTMODE_FLOPPY:
            strcat (line_ti, file_txt_bootfloppy_tm);
            break;
        case BOOTMODE_CD:
            strcat (line_ti, file_txt_bootcd_tm);
            break;
        case BOOTMODE_NET:
            strcat (line_ti, file_txt_bootnet_tm);
            break;
        case BOOTMODE_HARDDISK:
            strcat (line_ti, file_txt_bootharddisk_tm);
            break;
        case BOOTMODE_FTP:
            strcat (line_ti, file_txt_bootftp_tm);
            break;
        default:
            strcat (line_ti, "unknown");
            break;
        }
    strcat (line_ti, "\n");
    fprintf (file_pri, line_ti);

    if (bootmode_ig == BOOTMODE_HARDDISK)
        {
        fprintf (file_pri, "%s %s\n", file_txt_partition_tm, harddisk_tg);
        fprintf (file_pri, "%s %s\n", file_txt_fstyp_tm, fstype_tg);
        fprintf (file_pri, "%s %s\n", file_txt_serverdir_tm, server_dir_tg);
        }

    if (bootmode_ig == BOOTMODE_NET || bootmode_ig == BOOTMODE_FTP)
        {
        fprintf (file_pri, "%s %s\n", file_txt_netdevice_tm, netdevice_tg);

        fprintf (file_pri, "%s %s\n", file_txt_ip_tm, inet_ntoa (ipaddr_rg));

        if (plip_host_rg.s_addr)
            fprintf (file_pri, "%s %s\n", file_txt_pliphost_tm,
                                          inet_ntoa (plip_host_rg));
        else
            fprintf (file_pri, "%s %s\n", file_txt_netmask_tm,
                                          inet_ntoa (netmask_rg));
            
        if (gateway_rg.s_addr)
            fprintf (file_pri, "%s %s\n", file_txt_gateway_tm,
                                          inet_ntoa (gateway_rg));
        if (nameserver_rg.s_addr)
            fprintf (file_pri, "%s %s\n", file_txt_dnsserver_tm,
                                          inet_ntoa (nameserver_rg));
        fprintf (file_pri, "%s %s\n", file_txt_server_tm,
                 inet_ntoa (bootmode_ig == BOOTMODE_NET ? nfs_server_rg : ftp_server_rg));
        fprintf (file_pri, "%s %s\n", file_txt_serverdir_tm, server_dir_tg);

        if (machine_name_tg [0])
            fprintf (file_pri, "%s %s\n", file_txt_machine_name_tm,
                                          machine_name_tg);
        if (domain_name_tg [0])
            fprintf (file_pri, "%s %s\n", file_txt_domain_name_tm,
                                          domain_name_tg);
        }

    if (bootmode_ig == BOOTMODE_FTP)
        {
        if (ftp_user_tg [0])
            fprintf (file_pri, "%s %s\n", file_txt_ftp_user_tm,
                                          ftp_user_tg);
        if (ftp_password_tg [0])
            fprintf (file_pri, "%s %s\n", file_txt_ftp_passwd_tm,
                                          ftp_password_tg);
        if (ftp_proxy_tg [0])
            fprintf (file_pri, "%s %s\n", file_txt_ftp_proxy_tm,
                                          ftp_proxy_tg);
        if (ftp_proxyport_ig != -1)
            fprintf (file_pri, "%s %d\n", file_txt_ftp_proxy_port_tm,
                                          ftp_proxyport_ig);
        }

    mpar_write_modparms (file_pri);

    fclose (file_pri);

    file_pri = fopen ("/etc/mtab", "w");
    if (!file_pri)
        return;

    fprintf (file_pri, "/dev/initrd / minix rw 0 0\n");
    fprintf (file_pri, "none /proc proc rw 0 0\n");

    if (!ramdisk_ig)
        {
        switch (bootmode_ig)
            {
            case BOOTMODE_CD:
                sprintf (line_ti, "/dev/%s %s iso9660 ro 0 0\n", cdrom_tg, mountpoint_tg);
                break;
            case BOOTMODE_NET:
                sprintf (line_ti, "%s:%s %s nfs ro 0 0\n",
                                  inet_ntoa (nfs_server_rg),
                                  server_dir_tg, mountpoint_tg);
                break;
            default:
                line_ti [0] = 0;
                break;
            }
        }
    else
        sprintf (line_ti, "/dev/ram2 %s ext2 ro 0 0\n", inst_mountpoint_tg);

    fprintf (file_pri, line_ti);

    fclose (file_pri);
    }


int file_read_info (void)
    {
    char      filename_ti [MAX_FILENAME];
    FILE     *fd_pri;
    char      buffer_ti [MAX_X];
    window_t  win_ri;
    char      value_ti [MAX_X];


    dia_info (&win_ri, txt_get (TXT_SEARCH_INFOFILE));
    if (util_try_mount ("/dev/fd0", mountpoint_tg, MS_MGC_VAL | MS_RDONLY, 0))
        if (util_try_mount (floppy_tg, mountpoint_tg, MS_MGC_VAL | MS_RDONLY, 0))
            {
            win_close (&win_ri);
            return (-1);
            }

    sprintf (filename_ti, "%s/suse/setup/descr/info", mountpoint_tg);
    fd_pri = fopen (filename_ti, "r");
    if (!fd_pri)
        {
        umount (mountpoint_tg);
        win_close (&win_ri);
        return (-1);
        }

    while (fgets (buffer_ti, sizeof (buffer_ti) - 1, fd_pri))
        {
        fprintf (stderr, buffer_ti);
        file_trim_buffer (buffer_ti);
        file_get_value (buffer_ti, value_ti);

        if (!strncasecmp (file_txt_language_tm, buffer_ti,
                          strlen (file_txt_language_tm)))
            {
            if (!strncasecmp (value_ti, "english", 7))
                language_ig = LANG_ENGLISH;
            else if (!strncasecmp (value_ti, "german", 6))
                language_ig = LANG_GERMAN;
            else if (!strncasecmp (value_ti, "italian", 8))
                language_ig = LANG_ITALIAN;
            }

        if (!strncasecmp (file_txt_display_tm, buffer_ti,
                          strlen (file_txt_display_tm)))
            {
            if (!strncasecmp (value_ti, "Color", 5))
                color_ig = TRUE;
            else if (!strncasecmp (value_ti, "Mono", 4))
                color_ig = FALSE;
            }

        if (!strncasecmp (file_txt_keymap_tm, buffer_ti,
                          strlen (file_txt_keymap_tm)))
            strncpy (keymap_tg, value_ti, sizeof (keymap_tg));

        if (!strncasecmp (file_txt_bootmode_tm, buffer_ti,
                          strlen (file_txt_bootmode_tm)))
            {
            if (!strncasecmp (value_ti, file_txt_bootcd_tm,
                              strlen (file_txt_bootcd_tm)))
                bootmode_ig = BOOTMODE_CD;
            else if (!strncasecmp (value_ti, file_txt_bootharddisk_tm,
                                   strlen (file_txt_bootharddisk_tm)))
                bootmode_ig = BOOTMODE_HARDDISK;
            else if (!strncasecmp (value_ti, file_txt_bootnet_tm,
                                   strlen (file_txt_bootnet_tm)))
                bootmode_ig = BOOTMODE_NET;
            }

        if (!strncasecmp (file_txt_ip_tm, buffer_ti,
                          strlen (file_txt_ip_tm)))
            (void) net_check_address (value_ti, &ipaddr_rg);

        if (!strncasecmp (file_txt_netmask_tm, buffer_ti,
                          strlen (file_txt_netmask_tm)))
            (void) net_check_address (value_ti, &netmask_rg);

        if (!strncasecmp (file_txt_gateway_tm, buffer_ti,
                          strlen (file_txt_gateway_tm)))
            (void) net_check_address (value_ti, &gateway_rg);

        if (!strncasecmp (file_txt_server_tm, buffer_ti,
                          strlen (file_txt_server_tm)))
            (void) net_check_address (value_ti, &nfs_server_rg);

        if (!strncasecmp (file_txt_dnsserver_tm, buffer_ti,
                          strlen (file_txt_dnsserver_tm)))
            (void) net_check_address (value_ti, &nameserver_rg);

        if (!strncasecmp (file_txt_serverdir_tm, buffer_ti,
                          strlen (file_txt_serverdir_tm)))
            strncpy (server_dir_tg, value_ti, sizeof (server_dir_tg));

        if (!strncasecmp (file_txt_netdevice_tm, buffer_ti,
                          strlen (file_txt_netdevice_tm)))
            strncpy (netdevice_tg, value_ti, sizeof (netdevice_tg));

        if (!strncasecmp (file_txt_bootp_wait_tm, buffer_ti,
                          strlen (file_txt_bootp_wait_tm)))
            bootp_wait_ig = atoi (value_ti);
        }

    fclose (fd_pri);
    umount (mountpoint_tg);
    win_close (&win_ri);
    return (0);
    }


static void file_get_value (char *input_tv, char *value_tr)
    {
    char  *tmp_pci;

    tmp_pci = input_tv;
    while (*tmp_pci && *tmp_pci != ' ' && *tmp_pci != ':')
        tmp_pci++;

    if (*tmp_pci == ':')
        tmp_pci++;

    while (*tmp_pci && isblank (*tmp_pci))
        tmp_pci++;

    if (*tmp_pci)
        strcpy (value_tr, tmp_pci);
    else
        value_tr [0] = 0;
    }


static void file_trim_buffer (char *buffer_tr)
    {
    char  *tmp_pci;

    tmp_pci = strchr (buffer_tr, '#');
    if (tmp_pci)
        *tmp_pci = 0;

    tmp_pci = buffer_tr + strlen (buffer_tr) - 1;

    while (tmp_pci > buffer_tr &&
           (isblank (*tmp_pci) || *tmp_pci == 0x0a || *tmp_pci == '\n'))
        *(tmp_pci--) = 0;
    }
