/*
 *
 * linuxrc.c     Load modules and rootimage to ramdisk
 *
 * Copyright (c) 1996-1998  Hubert Mantel, S.u.S.E. GmbH  (mantel@suse.de)
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mount.h>
#include <linux/fs.h>
#include <linux/unistd.h>
#include <string.h>
#include <signal.h>
#include <stdarg.h>

#include "global.h"
#include "text.h"
#include "info.h"
#include "util.h"
#include "display.h"
#include "keyboard.h"
#include "dialog.h"
#include "window.h"
#include "module.h"
#include "net.h"
#include "pcmcia.h"
#include "install.h"
#include "settings.h"
#include "file.h"

extern int  insmod_main      (int argc, char **argv);
extern int  rmmod_main       (int argc, char **argv);
extern void cardmgr_main     (int argc, char **argv);
extern int  probe_main       (int argc, char **argv);
extern int  loadkeys_main    (unsigned int argc, char *argv[]);

static void lxrc_main_menu   (void);
static void lxrc_do_shell    (int argc, char **argv, char **env);
static void lxrc_init        (void);
static void lxrc_end         (void);
static int  lxrc_main_cb     (int what_iv);
static void lxrc_memcheck    (void);

static pid_t  lxrc_mempid_rm;
static int    lxrc_sig11_im = FALSE;


int main (int argc, char **argv, char **env)
    {
    int   rc_ii = 0;
    char *progname_pci;


    progname_pci = strrchr (argv [0], '/');
    if (progname_pci)
        progname_pci++;
    else
        progname_pci = argv [0];

    if (!strcmp (progname_pci, "sh"))
        lxrc_do_shell (argc, argv, env);
    else if (!strcmp (progname_pci, "insmod"))
        rc_ii = insmod_main (argc, argv);
    else if (!strcmp (progname_pci, "rmmod"))
        rc_ii = rmmod_main (argc, argv);
    else if (!strcmp (progname_pci, "loadkeys"))
        rc_ii = loadkeys_main ((unsigned int) argc, argv);
    else if (!strcmp (progname_pci, "cardmgr"))
        cardmgr_main (argc, argv);
    else if (!strcmp (progname_pci, "probe"))
        rc_ii = probe_main (argc, argv);
    else
        {
        if (argc > 1)
            testing_ig = TRUE;

        lxrc_init ();
        if (auto_ig)
            rc_ii = inst_auto_install ();
        else if (demo_ig)
            rc_ii = inst_start_demo ();

        if ((!auto_ig && !demo_ig) || rc_ii)
            lxrc_main_menu ();

        lxrc_end ();
        }

    return (rc_ii);
    }


int my_syslog (int type_iv, char *buffer_pci, ...)
    {
    va_list  args_ri;

    va_start (args_ri, buffer_pci);
    vfprintf (stderr, buffer_pci, args_ri);
    va_end (args_ri);
    fprintf (stderr, "\n");
    return (0);
    }


int my_logmessage (char *buffer_pci, ...)
    {
    va_list  args_ri;

    va_start (args_ri, buffer_pci);
    vfprintf (stderr, buffer_pci, args_ri);
    va_end (args_ri);
    fprintf (stderr, "\n");
    return (0);
    }


void lxrc_reboot (void)
    {
    if (dia_yesno (txt_get (TXT_ASK_REBOOT), 1) == YES)
        reboot (0xfee1dead, 672274793, 0x1234567);
    }


void lxrc_killall (void)
    {
    pid_t  mypid_ri;
    pid_t  i_ri;


    mypid_ri = getpid ();
    if (mypid_ri < 20)
        for (i_ri = 32767; i_ri > mypid_ri; i_ri--)
            if (i_ri != lxrc_mempid_rm)
                kill (i_ri, 9);
    }


/*
 *
 * Local functions
 *
 */

static void lxrc_catch_signal (int sig_iv)
    {
    if (sig_iv)
        {
        fprintf (stderr, "Caught signal %d!\n", sig_iv);
        sleep (10);
        }

    if (sig_iv == SIGBUS || sig_iv == SIGSEGV)
        lxrc_sig11_im = TRUE;

    signal (SIGHUP,  lxrc_catch_signal);
    signal (SIGBUS,  lxrc_catch_signal);
    signal (SIGINT,  lxrc_catch_signal);
    signal (SIGTERM, lxrc_catch_signal);
    signal (SIGSEGV, lxrc_catch_signal);
    signal (SIGPIPE, lxrc_catch_signal);
    }


static void lxrc_init (void)
    {
    int    i_ii;
    char  *language_pci;
    char  *linuxrc_pci;


    if (!testing_ig && getpid () > 19)
        {
        printf ("Seems we are on a running system; activating testmode...\n");
        sleep (3);
        testing_ig = TRUE;
        }

    if (txt_init ())
        {
        printf ("Corrupted texts!\n");
        exit (-1);
        }

    lxrc_catch_signal (0);
/*    reboot (0xfee1dead, 672274793, 0); */

    language_pci = getenv ("lang");
    if (language_pci)
        {
        if (!strncasecmp (language_pci, "english", 7))
            language_ig = LANG_ENGLISH;
        else if (!strncasecmp (language_pci, "german", 6))
            language_ig = LANG_GERMAN;
        else if (!strncasecmp (language_pci, "italian", 7))
            language_ig = LANG_ITALIAN;
        else if (!strncasecmp (language_pci, "french", 6))
            language_ig = LANG_FRENCH;
        }

    linuxrc_pci = getenv ("linuxrc");
    if (linuxrc_pci)
        {
        if (strstr (linuxrc_pci, "auto"))
            auto_ig = TRUE;
        
        if (strstr (linuxrc_pci, "demo"))
            demo_ig = TRUE;
        }

    kbd_init ();
    util_redirect_kmsg ();
    disp_init ();
    for (i_ii = 1; i_ii < max_y_ig; i_ii++)
        printf ("\n");
    printf ("[9;0]");
    freopen ("/dev/tty3", "a", stderr);

    (void) mount (0, "/proc", "proc", 0, 0);  
    disp_cursor_off ();
    info_init ();

    if (memory_ig > 25000000)
        force_ri_ig = TRUE;

    lxrc_memcheck ();

    util_print_banner ();

    if (auto_ig)
        file_read_info ();

    if (language_ig == LANG_UNDEF)
        set_choose_language ();
    set_choose_display ();
    util_print_banner ();
    set_choose_keytable ();
    util_update_kernellog ();
    }


static void lxrc_end (void)
    {
    kill (lxrc_mempid_rm, 9);
    lxrc_killall ();
    printf ("[9;15]");
/*    reboot (0xfee1dead, 672274793, 0x89abcdef); */
    mod_free_modules ();
    (void) umount (mountpoint_tg);
    (void) umount ("/proc");
    disp_cursor_on ();
    kbd_end ();
    disp_end ();
    }


static int lxrc_main_cb (int what_iv)
    {
    int    rc_ii = 1;


    switch (what_iv)
        {
        case 1:
            rc_ii = set_settings ();
            if (rc_ii == 1 || rc_ii == 2)
                rc_ii = 0;
            else if (rc_ii == 0)
                rc_ii = 1;
            break;
        case 2:
            info_menu ();
            break;
        case 3:
            mod_menu ();
            break;
        case 4:
            rc_ii = inst_menu ();
            break;
        case 5:
            lxrc_reboot ();
        default:
            break;
        }

    return (rc_ii);
    }


static void lxrc_main_menu (void)
    {
    int    width_ii = 40;
    item_t items_ari [5];
    int    i_ii;
    int    choice_ii;
    int    nr_items_ii = sizeof (items_ari) / sizeof (items_ari [0]);;


    auto_ig = FALSE;

    util_create_items (items_ari, nr_items_ii, width_ii);

    do
        {
        strcpy (items_ari [0].text, txt_get (TXT_SETTINGS));
        strcpy (items_ari [1].text, txt_get (TXT_MENU_INFO));
        strcpy (items_ari [2].text, txt_get (TXT_MENU_MODULES));
        strcpy (items_ari [3].text, txt_get (TXT_MENU_START));
        strcpy (items_ari [4].text, txt_get (TXT_END_REBOOT));
        for (i_ii = 0; i_ii < nr_items_ii; i_ii++)
            {
            util_center_text (items_ari [i_ii].text, width_ii);
            items_ari [i_ii].func = lxrc_main_cb;
            }

        choice_ii = dia_menu (txt_get (TXT_HDR_MAIN), items_ari,
                              nr_items_ii, 2);
        if (!choice_ii)
            if (dia_message (txt_get (TXT_NO_LXRC_END), MSGTYPE_INFO) == -42)
                choice_ii = 42;

        if (choice_ii == 1)
            {
            util_print_banner ();
            choice_ii = 0;
            }
        }
    while (!choice_ii);

    util_free_items (items_ari, nr_items_ii);
    }


static void lxrc_do_shell (int argc, char **argv, char **env)
    {
    char  command_ati [10][100];
    char  progname_ti [100];
    int   i_ii = 0;
    int   j_ii = 0;
    int   k_ii = 0;
    char *arguments_apci [10];

    while (argv [2][i_ii] == ' ')
        i_ii++;

    do
        {
        command_ati [j_ii][k_ii++] = argv [2][i_ii++];
        if (argv [2][i_ii] == ' ' || argv [2][i_ii] == 0)
            {
            command_ati [j_ii][k_ii] = 0;
            arguments_apci [j_ii] = command_ati [j_ii];
            j_ii++;
            k_ii = 0;

            while (argv [2][i_ii] == ' ')
                i_ii++;
            }
        }
    while (argv [2][i_ii]);

    arguments_apci [j_ii] = 0;
    freopen ("/dev/tty3", "a", stdout);
    freopen ("/dev/tty3", "a", stderr);
    printf ("Executing: \"%s\"\n", argv [2]);

    sprintf (progname_ti, "/bin/%s", arguments_apci [0]);
    execve (progname_ti, arguments_apci, env);
    exit (0);
    }


static void lxrc_memcheck (void)
    {
    int   nr_pages_ii;
    int   max_pages_ii;
    char *data_pci;
    int   i_ii;


    lxrc_mempid_rm = fork ();
    if (lxrc_mempid_rm)
        return;

    lxrc_catch_signal (0);

    if (memory_ig < 8000000)
        max_pages_ii = 40;
    else
        max_pages_ii = 100;

    while (1)
        {
        nr_pages_ii = 1 + (int) ((double) max_pages_ii * rand () / (RAND_MAX + 1.0));
        data_pci = malloc (nr_pages_ii * 4096);
        if (data_pci)
            {
            for (i_ii = 0; i_ii < nr_pages_ii && !lxrc_sig11_im; i_ii++)
                {
                data_pci [i_ii * 4096] = 42;
                usleep (10000);
                }
            lxrc_sig11_im = FALSE;
            free (data_pci);
            }
        sleep (1);
        }
    }
