/*
 *
 * module.c      Load modules needed for installation
 *
 * Copyright (c) 1996-1998  Hubert Mantel, S.u.S.E. GmbH  (mantel@suse.de)
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/mount.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/fs.h>

#include "global.h"
#include "text.h"
#include "module.h"
#include "util.h"
#include "dialog.h"
#include "display.h"
#include "window.h"
#include "modparms.h"
#include "pcmcia.h"
#include "rootimage.h"
#include "net.h"
#include "info.h"
#include "keyboard.h"

static module_t mod_scsi_mod_arm [] = 
{
{ ID_AHA_2940,   "Adaptec 274x/284x/294x STANDARD",    "aic7xxx",    0 },
{ ID_AHA_2940O,  "Adaptec 274x/284x/294x (4.1.1)",     "aic7xxx-old",0 },
{ ID_AHA_2940N,  "Adaptec 274x/284x/294x (5.1.0) BETA","aic7xxx-new",0 },
{ ID_NCR_53C8XX, "NCR 53C8XX (BSD driver)",            "ncr53c8xx",  0 },
{ ID_BUSLOGIC,   "BusLogic",                           "BusLogic",   0 },
{ ID_AHA_154x,   "Adaptec 154x",                       "aha1542",    0 },
{ ID_AHA_152x,   "Adaptec 1505/151x/152x/2825",        "aha152x",    "aha152x=0x340,11,7,0,0,0,100,0" },
{ ID_AHA_1740,   "Adaptec 1740",                       "aha1740",    0 },
{ ID_ULTRAST_14, "UltraStor 14F/34F",                  "u14-34f",    0 },
{ ID_ULTRASTOR,  "UltraStor (alternative driver)",     "ultrastor",  0 },
{ ID_ADVANSYS,   "AdvanSys",                           "advansys",   "asc_iopflag=1 asc_ioport=0x110" },
{ ID_FDOMAIN,    "Future Domain 16xx",                 "fdomain",    "port_base=0x340 setup_called=1" },
{ ID_IN2000,     "Always IN 2000",                     "in2000",     0 },
{ ID_NCR_5380,   "Generic NCR5380/53c400",             "g_NCR5380",  0 },
{ ID_NCR_53C406, "NCR53c406a",                         "NCR53c406a", 0 },
{ ID_QLOGIC_FAS, "Qlogic FAS",                         "qlogicfas",  0 },
{ ID_QLOGIC_ISP, "Qlogic ISP",                         "qlogicisp",  0 },
{ ID_PAS16,      "Pro Audio Spectrum SCSI",            "pas16",      0 },
{ ID_SEAGATE,    "Seagate ST-02/Fut. Domain TMC-8xx",  "seagate",    0 },
{ ID_TRANTOR,    "Trantor T128/T128F/T228",            "t128",       0 },
{ ID_DTC,        "DTC 3180/3280",                      "dtc",        0 },
{ ID_EATA_DMA,   "EATA-DMA (DPT, NEC, AT&T, SNI etc)", "eata_dma",   0 },
{ ID_EATA_PIO,   "EATA-PIO (old DPT PM2001, PM2012A)", "eata_pio",   0 },
{ ID_WD7000,     "Western Digital 7000 FASST",         "wd7000",     0 },
{ ID_EATA_ISA,   "EATA (E)ISA (PM2011/021/012 etc)",   "eata",       0 },
{ ID_AM53C974,   "AM53/79C974",                        "AM53C974",   0 },
{ ID_IOMEGA,     "IOMEGA Parallel Port ZIP drive",     "ppa",        "ppa_base=0x378" },
{ ID_GDTH,       "GDT SCSI Disk Array (ICP Vortex)",   "gdth",       0 },
{ ID_DC390T,     "Tekram DC390(T) (AM53C974A chip)",   "tmscsim",    0 },
{ ID_53C78XX,    "NCR 53c7,8xx (old driver)",          "53c7,8xx",   0 },
{ ID_INI9100,    "Initio 9100, 9100W",                 "ini9100",    0 },
{ ID_INI9100U,   "Initio Ultra controllers",           "ini9100u",   0 }
};

static module_t mod_cdrom_mod_arm [] = 
{
{ ID_AZTECH,     "Aztech/Orchid/Okano/Wearnes/TXC",    "aztcd",      "aztcd=0x320" },
{ ID_GOLDSTAR,   "Goldstar R420",                      "gscd",       "gscd=0x340" },
{ ID_MCD,        "Mitsumi (standard)",                 "mcd",        "mcd=0x300,10" },
{ ID_MCDX,       "Mitsumi (XA/MultiSession)",          "mcdx",       "mcdx=0x300,10" },
{ ID_OPTCD,      "Optics Storage DOLPHIN 8000AT",      "optcd",      "optcd=0x340" },
{ ID_CM206,      "Philips/LMS CM206",                  "cm206",      "cm206=0x340,11" },
{ ID_SANYO,      "Sanyo CDR-H94A",                     "sjcd",       "sjcd_base=0x340" },
{ ID_ISP16,      "ISP16/MAD16/Mozart",                 "isp16",      "isp16_cdrom_base=0x340 isp16_cdrom_irq=0 isp16_cdrom_dma=0 isp16_cdrom_type=Sanyo" },
{ ID_CDU31A,     "Sony CDU31A/CDU33A",                 "cdu31a",     "cdu31a_port=0x340 cdu31a_irq=0" },
{ ID_SONY535,    "Sony CDU535",                        "sonycd535",  "sonycd535=0x340" },
{ ID_SBPCD,      "SBPCD (Matsushita/Panasonic etc.)",  "sbpcd",      "sbpcd=0x230,1" },
{ ID_ATEN,       "ATEN EH-100 parallel port",          "aten",       0 },
{ ID_BPCK,       "MicroSolutions parallel port",       "bpck",       0 },
{ ID_COMM,       "DataStor (old) parallel port",       "comm",       0 },
{ ID_DSTR,       "DataStor EP2000 parallel port",      "dstr",       0 },
{ ID_EPAT,       "Shuttle EPAT parallel port",         "epat",       0 },
{ ID_EPIA,       "Shuttle EPIA parallel port",         "epia",       0 },
{ ID_FIT2,       "FIT TD-2000 parallel port",          "fit2",       0 },
{ ID_FIT3,       "FIT TD-3000 parallel port",          "fit3",       0 },
{ ID_FRPW,       "Freecom Power parallel port",        "frpw",       0 },
{ ID_KBIC,       "KBIC-951A/971A parallel port",       "kbic",       0 },
{ ID_KTTI,       "KT Technology parallel port",        "ktti",       0 },
{ ID_ON20,       "OnSpec 90c20 parallel port",         "on20",       0 },
{ ID_ON26,       "OnSpec 90c26 parallel port",         "on26",       0 }
};

static module_t mod_net_mod_arm [] =
{
{ ID_3C501,      "3Com 3c501",                         "3c501",      "io=0x280" },
{ ID_3C503,      "3Com 3c503",                         "3c503",      "io=0x280" },
{ ID_3C505,      "3Com 3c505",                         "3c505",      "io=0x300" },
{ ID_3C507,      "3Com 3c507",                         "3c507",      "io=0x300" },
{ ID_3C509,      "3Com 3c509/3c579",                   "3c509",      0 },
{ ID_3C515,      "3Com 3c515",                         "3c515",      "options=0" },
{ ID_3C590,      "3Com 3c59x/3c90x",                   "3c59x",      "options=0"},
{ ID_AC3200,     "Ansel Communications EISA 3200",     "ac3200",     "io=0x300 irq=10 mem=0xd0000" },
{ ID_APRICOT,    "Apricot Xen-II on board ethernet",   "apricot",    "io=0x300" },
{ ID_ARCNET,     "ARCnet",                             "arcnet",     "io=0x300" },
{ ID_AT1700,     "AT1700",                             "at1700",     "io=0x260" },
{ ID_ATP,        "AT-LAN-TEC/RealTek pocket adaptor",  "atp",        0 },
{ ID_DE_45,      "DE425, DE434, DE435, DE450, DE500",  "de4x5",      "io=0x100" },
{ ID_DE600,      "D-Link DE600 pocket adaptor",        "de600",      0        },
{ ID_DE620,      "D-Link DE620 pocket adaptor",        "de620",      "io=0x378 irq=7 bnc=1" },
{ ID_DEPCA,      "DEPCA,DE10x,DE200,DE201,DE202,DE422","depca",      "io=0x200" },
{ ID_DGRS,       "Digi Intl. RightSwitch SE-X",        "dgrs",       0 },
{ ID_E2100,      "Cabletron E21xx",                    "e2100",      "io=0x300" },
{ ID_EEPRO,      "Intel EtherExpressPro",              "eepro",      "io=0x260" },
{ ID_EEPRO100,   "Intel EtherExpress Pro 100",         "eepro100",   0          },
{ ID_EEXPRESS,   "Intel EtherExpress 16",              "eexpress",   "io=0x300" },
{ ID_EPIC100,    "SMC 83c170 EPIC/100",                "epic100",    0          },
{ ID_ETH16I,     "ICL EtherTeam 16i/32",               "eth16i",     "io=0x2a0" },
{ ID_EWORK,      "EtherWORKS 3 (DE203, DE204, DE205)", "ewrk3",      "io=0x300" },
{ ID_FMV18X,     "FMV-181/182/183/184",                "fmv18x",     "io=0x220" },
{ ID_HP_PLUS,    "HP PCLAN+ (27247B and 27252A)",      "hp-plus",    "io=0x300" },
{ ID_HP_PCLAN,   "HP PCLAN (27245 / 27xxx)",           "hp",         "io=0x300" },
{ ID_HP_100,     "HP 10/100VG PCLAN (ISA, EISA, PCI)", "hp100",      "hp100_port=0x380" },
{ ID_TOKEN,      "IBM Tropic chipset token ring",      "ibmtr",      "io=0xa20" },
{ ID_LANCE,      "AMD LANCE and PCnet (AT1500/NE2100)","lance",      0 },
{ ID_NE2000,     "NE 2000 / NE 1000 (ISA)",            "ne",         "io=0x300" },
{ ID_NE2K_PCI,   "NE 2000 (PCI)",                      "ne2k-pci",   0 },
{ ID_NI5210,     "NI5210",                             "ni52",       "io=0x360 irq=9 memstart=0xd0000 memend=0xd4000" },
{ ID_NI6510,     "ni6510 (am7990 \"lance\" chip)",     "ni65",       "io=0x360 irq=9 dma=0" },
{ ID_PCNET32,    "AMD PCI PCnet32 (PCI bus NE2100)",   "pcnet32",    0 },
{ ID_PLIP,       "PLIP (IP via parallel port)",        "plip",       "io=0x378 irq=7"  },
{ ID_RTL8139,    "RealTek RTL8129/8139",               "rtl8139",    0          },
{ ID_WD_ULTRA,   "SMC Ultra",                          "smc-ultra",  "io=0x200" },
{ ID_ULTRA32,    "SMC Ultra32",                        "smc-ultra32",0 },
{ ID_WD_9194,    "SMC 9194",                           "smc9194",    "io=0x200" },
{ ID_TLAN,       "Compaq Netelligent 10/100/NetFlex 3","tlan",       0          },
{ ID_TULIP,      "DECchip Tulip (dc21x4x) PCI",        "tulip",      "options=0" },
{ ID_WD_80X3,    "Western Digital WD80x3",             "wd",         0 },
{ ID_YELLOWFIN,  "Packet Engines Yellowfin Gigabit",   "yellowfin",  0 }
};

static enum modid_t mod_noauto_arm [] =
{
  ID_SBPCD,
  ID_WD7000,
  ID_CDU31A,
  ID_3C501,
  ID_PLIP,
  ID_IOMEGA,
  ID_AZTECH,
  ID_DE600,
  ID_DE620,
  ID_E2100,
  ID_AC3200,
  ID_NCR_5380,
  ID_NCR_53C810,
  ID_EEPRO100,
  ID_TLAN,
  ID_ATEN,
  ID_BPCK,
  ID_COMM,
  ID_DSTR,
  ID_EPAT,
  ID_EPIA,
  ID_FIT2,
  ID_FIT3,
  ID_FRPW,
  ID_KBIC,
  ID_KTTI,
  ID_ON20,
  ID_ON26,
  ID_ATP,
  ID_ADVANSYS
};

static enum modid_t mod_is_ppcd_arm [] =
{
  ID_ATEN,
  ID_BPCK,
  ID_COMM,
  ID_DSTR,
  ID_EPAT,
  ID_EPIA,
  ID_FIT2,
  ID_FIT3,
  ID_FRPW,
  ID_KBIC,
  ID_KTTI,
  ID_ON20,
  ID_ON26
};

#define NR_MODULES          150

static int       mod_ram_modules_im = FALSE;
static char     *mod_modpath_tm = "/modules";
static module_t  mod_current_arm [NR_MODULES];
static int       mod_show_kernel_im = FALSE;

#define NR_SCSI_MODULES     (sizeof(mod_scsi_mod_arm)/sizeof(mod_scsi_mod_arm[0]))
#define NR_CDROM_MODULES    (sizeof(mod_cdrom_mod_arm)/sizeof(mod_cdrom_mod_arm[0]))
#define NR_NET_MODULES      (sizeof(mod_net_mod_arm)/sizeof(mod_net_mod_arm[0]))
#define NR_NO_AUTOPROBE     (sizeof(mod_noauto_arm)/sizeof(mod_noauto_arm[0]))
#define NR_PPCD             (sizeof(mod_is_ppcd_arm)/sizeof(mod_is_ppcd_arm[0]))
#define MENU_WIDTH          55


static void      mod_autoload         (void);
static int       mod_try_auto         (module_t *module_prv,
                                       window_t *status_prv);
static int       mod_auto_allowed     (enum modid_t id_iv);
static module_t *mod_get_description  (char *name_tv);
static void      mod_delete_module    (void);
static int       mod_menu_cb          (int what_iv);
static int       mod_choose_cb        (int what_iv);
static void      mod_load_core        (void);
static int       mod_get_current_list (int mod_type_iv, int *nr_modules_pir,
                                       int *more_pir);
static int       mod_is_ppcd          (char *name_tv);
static int       mod_load_ppcd_core   (void);


void mod_menu (void)
    {
    item_t  items_ari [7];
    int     nr_items = sizeof (items_ari) / sizeof (items_ari [0]);
    int     i_ii;


    net_stop ();
    mod_load_core ();
    util_create_items (items_ari, nr_items, 40);
    strcpy (items_ari [0].text, txt_get (TXT_LOAD_SCSI));
    strcpy (items_ari [1].text, txt_get (TXT_LOAD_CDROM));
    strcpy (items_ari [2].text, txt_get (TXT_LOAD_NET));
    strcpy (items_ari [3].text, txt_get (TXT_LOAD_PCMCIA));
    strcpy (items_ari [4].text, txt_get (TXT_SHOW_MODULES));
    strcpy (items_ari [5].text, txt_get (TXT_DEL_MODULES));
    strcpy (items_ari [6].text, txt_get (TXT_AUTO_LOAD));
    for (i_ii = 0; i_ii < nr_items; i_ii++)
        {
        util_center_text (items_ari [i_ii].text, 40);
        items_ari [i_ii].func = mod_menu_cb;
        }

    (void) dia_menu (txt_get (TXT_MENU_MODULES), items_ari,
                     nr_items, 1);

    util_free_items (items_ari, nr_items);
    }


int mod_load_module (char *module_tv, char *params_tv)
    {
    char  command_ti [300];
    int   rc_ii;


    sprintf (command_ti, "insmod %s/%s.o ", mod_modpath_tm, module_tv);
    if (params_tv && params_tv [0])
        strcat (command_ti, params_tv);

    if (mod_show_kernel_im)
        kbd_switch_tty (4);

    rc_ii = system (command_ti);

    if (mod_show_kernel_im)
        kbd_switch_tty (1);

    util_update_kernellog ();
    return (rc_ii);
    }


void mod_unload_module (char *module_tv)
    {
    char  command_ti [300];

    sprintf (command_ti, "rmmod %s", module_tv);
    system (command_ti);
    util_update_kernellog ();
    }


void mod_show_modules (void)
    {
    char     *items_ari [20];
    int       i_ii;
    int       nr_items_ii = 0;
    char      buffer_ti [MAX_X];
    FILE     *fd_pri;
    module_t *module_pri;


    fd_pri = fopen ("/proc/modules", "r");
    if (!fd_pri)
        return;

    while (fgets (buffer_ti, MAX_X - 1, fd_pri) &&
           nr_items_ii < sizeof (items_ari) / sizeof (items_ari [0]))
        {
        i_ii = 0;
        while (buffer_ti [i_ii] != ' ')
            i_ii++;
        buffer_ti [i_ii] = 0;
        module_pri = mod_get_description (buffer_ti);
        if (module_pri)
            {
            items_ari [nr_items_ii] = malloc (MENU_WIDTH);
            strncpy (items_ari [nr_items_ii],
                     module_pri->description, MENU_WIDTH);
            util_fill_string (items_ari [nr_items_ii], MENU_WIDTH - 4);
            nr_items_ii++;
            }
        }
    fclose (fd_pri);

    if (!nr_items_ii)
        (void) dia_message (txt_get (TXT_NO_MODULES), MSGTYPE_INFO);
    else
        {
        dia_show_lines (txt_get (TXT_SHOW_MODULES), items_ari, nr_items_ii,
                        MENU_WIDTH, FALSE);

        for (i_ii = 0; i_ii < nr_items_ii; i_ii++)
            free (items_ari [i_ii]);
        }
    }


void mod_free_modules (void)
    {
    if (mod_ram_modules_im)
        {
        umount (mod_modpath_tm);
        util_free_ramdisk ("/dev/ram2");
        mod_ram_modules_im = FALSE;
        }
    }


int mod_load_by_user (int mod_type_iv)
    {
    int     nr_modules_ii;
    char   *header_ti;
    item_t  items_ari [NR_MODULES];
    int     i_ii;
    int     rc_ii;
    int     width_ii = MENU_WIDTH;
    int     choice_ii;
    int     more_ii;
    int     ready_ii = FALSE;


    switch (mod_type_iv)
        {
        case MOD_TYPE_SCSI:
        default:
            header_ti = txt_get (TXT_LOAD_SCSI);
            break;
        case MOD_TYPE_OTHER:
            header_ti = txt_get (TXT_LOAD_CDROM);
            break;
        case MOD_TYPE_NET:
            header_ti = txt_get (TXT_LOAD_NET);
            break;
        }

    do
        {
        rc_ii = mod_get_ram_modules (mod_type_iv);
        if (rc_ii)
            return (0);

        rc_ii = mod_get_current_list (mod_type_iv, &nr_modules_ii, &more_ii);
        if (rc_ii || !nr_modules_ii)
            return (0);

        for (i_ii = 0; i_ii < nr_modules_ii; i_ii++)
            {
            items_ari [i_ii].text = malloc (width_ii);
            sprintf (items_ari [i_ii].text, "%14s : %s",
                     mod_current_arm [i_ii].module_name,
                     mod_current_arm [i_ii].description);
            util_fill_string (items_ari [i_ii].text, width_ii);
            items_ari [i_ii].func = mod_choose_cb;
            }

        if (more_ii)
            {
            items_ari [nr_modules_ii].text = malloc (width_ii);
            items_ari [nr_modules_ii].func = 0;
            strcpy (items_ari [nr_modules_ii].text, txt_get (TXT_MORE_MODULES));
            util_fill_string (items_ari [nr_modules_ii++].text, width_ii);
            }

        choice_ii = dia_menu (header_ti, items_ari, nr_modules_ii, 1);
        
        util_free_items (items_ari, nr_modules_ii);

        if (choice_ii == nr_modules_ii && more_ii)
            {
            if (dia_message (txt_get (TXT_ENTER_MODDISK), MSGTYPE_INFO) == -1)
                {
                ready_ii = TRUE;
                choice_ii = 0;
                }
            else
                mod_free_modules ();
            }
        else
            ready_ii = TRUE;
        }
    while (!ready_ii);

    return (choice_ii);
    }


int mod_get_ram_modules (int type_iv)
    {
    char      testfile_ti [MAX_FILENAME];
    char     *modfile_pci;
    int       rc_ii = 0;


    strcpy (testfile_ti, mod_modpath_tm);
    if (type_iv == MOD_TYPE_SCSI)
        {
        strcat (testfile_ti, "/SCSI");
        modfile_pci = "scsi-mod.gz";
        }
    else if (type_iv == MOD_TYPE_NET)
        {
        strcat (testfile_ti, "/NET");
        modfile_pci = "net-mod.gz";
        }
    else
        {
        strcat (testfile_ti, "/OTHER");
        modfile_pci = "other-mod.gz";
        }

    if (!util_check_exist (testfile_ti))
        {
        if (util_try_mount ("/dev/fd0", mountpoint_tg, MS_MGC_VAL | MS_RDONLY, 0))
            if (util_try_mount (floppy_tg, mountpoint_tg, MS_MGC_VAL | MS_RDONLY, 0))
                {
                dia_message (txt_get (TXT_ERROR_READ_DISK), MSGTYPE_ERROR);
                return (-1);
                }

        sprintf (testfile_ti, "%s/%s", mountpoint_tg, modfile_pci);
        rc_ii = root_load_rootimage (testfile_ti);
        umount (mountpoint_tg);
        if (!rc_ii)
            {
            rc_ii = util_try_mount (RAMDISK_2, mod_modpath_tm, MS_MGC_VAL | MS_RDONLY, 0);

            if (rc_ii)
                dia_message (txt_get (TXT_ERROR_READ_DISK), MSGTYPE_ERROR);

            mod_ram_modules_im = TRUE;
            }
        }

    return (rc_ii);
    }


int mod_auto (int type_iv)
    {
    int       nr_modules_ii;
    int       i_ii;
    window_t  win_ri;
    int       rc_ii;
    int       success_ii;
    int       dummy_ii;


    rc_ii = mod_get_ram_modules (type_iv);
    if (rc_ii)
        return (1);

    rc_ii = mod_get_current_list (type_iv, &nr_modules_ii, &dummy_ii);
    if (rc_ii)
        return (1);

    dia_status_on (&win_ri, "");
    success_ii = FALSE;
    i_ii = 0;

    while (i_ii < nr_modules_ii && !success_ii)
        {
        dia_status (&win_ri, ((i_ii + 1) * 100) / nr_modules_ii);
        if (!mod_try_auto (&mod_current_arm [i_ii], &win_ri))
            success_ii = TRUE;
        else
            i_ii++;
        }

    win_close (&win_ri);
    if (success_ii)
        return (0);
    else
        return (-1);
    }


/*
 *
 * Local functions
 *
 */

static int mod_choose_cb (int what_iv)
    {
    char      text_ti [200];
    int       rc_ii = 1;
    char      params_ti [MAX_PARAM_LEN];
    char      pcd_params_ti [MAX_PARAM_LEN];
    window_t  win_ri;
    module_t  pcd_ri = { ID_PPCD, "", "pcd", "drive0=0x378" };


    if (mod_is_ppcd (mod_current_arm [what_iv - 1].module_name))
        {
        rc_ii = mod_load_ppcd_core ();
        if (rc_ii)
            return (rc_ii);
        }

    if (mod_current_arm [what_iv - 1].example)
        strcpy (params_ti, mod_current_arm [what_iv - 1].example);
    else
        params_ti [0] = 0;

    if (!mpar_get_params (&mod_current_arm [what_iv - 1], params_ti))
        {
        sprintf (text_ti, txt_get (TXT_TRY_TO_LOAD),
                 mod_current_arm [what_iv - 1].module_name);

        dia_info (&win_ri, text_ti);
        rc_ii = mod_load_module (mod_current_arm [what_iv - 1].module_name,
                                 params_ti);
        if (!rc_ii && mod_is_ppcd (mod_current_arm [what_iv - 1].module_name))
            {
            win_close (&win_ri);
            pcd_params_ti [0] = 0;
            mpar_get_params (&pcd_ri, pcd_params_ti);
            sprintf (text_ti, txt_get (TXT_TRY_TO_LOAD), "pcd");
            dia_info (&win_ri, text_ti);
            rc_ii = mod_load_module ("pcd", pcd_params_ti);
            if (!rc_ii)
                {
                mpar_save_modparams ("pcd", pcd_params_ti);
                strcpy (cdrom_tg, "pcd0");
                strcpy (ppcd_tg, mod_current_arm [what_iv - 1].module_name);
                }
            }

        win_close (&win_ri);

        if (rc_ii == 0)
            {
            sprintf (text_ti, txt_get (TXT_LOAD_SUCCESSFUL),
                     mod_current_arm [what_iv - 1].module_name);
            (void) dia_message (text_ti, MSGTYPE_INFO);
            mpar_save_modparams (mod_current_arm [what_iv - 1].module_name,
                                 params_ti);
            }
        else
            {
            if (mod_is_ppcd (mod_current_arm [what_iv - 1].module_name))
                mod_unload_module (mod_current_arm [what_iv - 1].module_name);

            util_beep (FALSE);
            sprintf (text_ti, txt_get (TXT_LOAD_FAILED),
                     mod_current_arm [what_iv - 1].module_name);
            (void) dia_message (text_ti, MSGTYPE_ERROR);
            }

        (void) dia_show_file (txt_get (TXT_INFO_KERNEL), lastlog_tg, TRUE);
        }

    if (rc_ii)
        return (what_iv);
    else
        return (0);
    }


static int mod_try_auto (module_t *module_prv, window_t *status_prv)
    {
    char   text_ti [STATUS_SIZE];
    int    rc_ii = -1;


    memset (text_ti, ' ', STATUS_SIZE);
    sprintf (text_ti + 14, txt_get (TXT_AUTO_RUNNING), module_prv->module_name);
    disp_set_color (status_prv->fg_color, status_prv->bg_color);
    win_print (status_prv, 2, 2, text_ti);
    fflush (stdout);

    if (mod_auto_allowed (module_prv->id) ||
        (demo_ig && mod_current_arm == mod_cdrom_mod_arm))
        rc_ii = mod_load_module (module_prv->module_name, module_prv->example);

    if (rc_ii == 0)
        mpar_save_modparams (module_prv->module_name, module_prv->example);

    return (rc_ii);
    }


static void mod_autoload (void)
    {
    int       i_ii;
    int       nr_iv = 0;
    window_t  win_ri;
    char      text_ti [200];
    int       rc_ii;


    dia_status_on (&win_ri, "");
    rc_ii = mod_get_ram_modules (MOD_TYPE_SCSI);
    if (rc_ii)
        {
        win_close (&win_ri);
        return;
        }
    for (i_ii = 0; i_ii < NR_SCSI_MODULES; i_ii++)
        {
        dia_status (&win_ri, (nr_iv++ * 100) / NR_MODULES);
        if (!mod_try_auto (&mod_scsi_mod_arm [i_ii], &win_ri))
            {
            if (!scsi_tg [0])
                strcpy (scsi_tg, mod_scsi_mod_arm [i_ii].module_name);
            sprintf (text_ti, txt_get (TXT_LOAD_SUCCESSFUL),
                     mod_scsi_mod_arm [i_ii].module_name);
            strcat (text_ti, "\n\n");
            strcat (text_ti, txt_get (TXT_WANT_MORE_SCSI));
            if (dia_yesno (text_ti, NO) != YES)
                {
                i_ii = NR_SCSI_MODULES;
                nr_iv = NR_SCSI_MODULES;
                }
            }
        }

    rc_ii = mod_get_ram_modules (MOD_TYPE_NET);
    if (rc_ii)
        {
        win_close (&win_ri);
        return;
        }
    for (i_ii = 0; i_ii < NR_NET_MODULES; i_ii++)
        {
        dia_status (&win_ri, (nr_iv++ * 100) / NR_MODULES);
        if (!mod_try_auto (&mod_net_mod_arm [i_ii], &win_ri))
            {
            if (!net_tg [0])
                strcpy (net_tg, mod_net_mod_arm [i_ii].module_name);
            sprintf (text_ti, txt_get (TXT_LOAD_SUCCESSFUL),
                     mod_net_mod_arm [i_ii].module_name);
            strcat (text_ti, "\n\n");
            strcat (text_ti, txt_get (TXT_WANT_MORE_NET));
            if (dia_yesno (text_ti, NO) != YES)
                {
                i_ii = NR_NET_MODULES;
                nr_iv = NR_SCSI_MODULES + NR_NET_MODULES;
                }
            }
        }

    rc_ii = mod_get_ram_modules (MOD_TYPE_OTHER);
    if (rc_ii)
        {
        win_close (&win_ri);
        return;
        }
    for (i_ii = 0; i_ii < NR_CDROM_MODULES; i_ii++)
        {
        dia_status (&win_ri, (nr_iv++ * 100) / NR_MODULES);
        if (!mod_try_auto (&mod_cdrom_mod_arm [i_ii], &win_ri))
            {
            if (!cdrom_tg [0])
                strcpy (cdrom_tg, mod_cdrom_mod_arm [i_ii].module_name);
            sprintf (text_ti, txt_get (TXT_LOAD_SUCCESSFUL),
                     mod_cdrom_mod_arm [i_ii].module_name);
            strcat (text_ti, "\n\n");
            strcat (text_ti, txt_get (TXT_WANT_MORE_CDROM));
            if (dia_yesno (text_ti, NO) != YES)
                i_ii = NR_CDROM_MODULES;
            }
        }

    win_close (&win_ri);
    (void) dia_show_file (txt_get (TXT_INFO_KERNEL), kernellog_tg, TRUE);
    mod_show_modules ();
    }


static int mod_auto_allowed (enum modid_t id_iv)
    {
    int  i_ii = 0;
    int  found_ii = FALSE;

    while (i_ii < NR_NO_AUTOPROBE && !found_ii)
        if (id_iv == mod_noauto_arm [i_ii])
            found_ii = TRUE;
        else
            i_ii++;

    return (!found_ii);
    }


static int mod_is_ppcd (char *name_tv)
    {
    int        i_ii = 0;
    int        found_ii = FALSE;
    module_t  *module_pri;


    module_pri = mod_get_description (name_tv);
    if (!module_pri)
        return (FALSE);

    while (i_ii < NR_PPCD && !found_ii)
        if (module_pri->id == mod_is_ppcd_arm [i_ii])
            found_ii = TRUE;
        else
            i_ii++;

    return (found_ii);
    }


static module_t *mod_get_description (char *name_tv)
    {
    int  i_ii;
    int  found_ii = FALSE;

    i_ii = 0;
    while (i_ii < NR_SCSI_MODULES && !found_ii)
        if (!strcmp (name_tv, mod_scsi_mod_arm [i_ii].module_name))
            found_ii = TRUE;
        else
            i_ii++;

    if (found_ii)
        return (&mod_scsi_mod_arm [i_ii]);

    i_ii = 0;
    while (i_ii < NR_CDROM_MODULES && !found_ii)
        if (!strcmp (name_tv, mod_cdrom_mod_arm [i_ii].module_name))
            found_ii = TRUE;
        else
            i_ii++;

    if (found_ii)
        return (&mod_cdrom_mod_arm [i_ii]);

    i_ii = 0;
    while (i_ii < NR_NET_MODULES && !found_ii)
        if (!strcmp (name_tv, mod_net_mod_arm [i_ii].module_name))
            found_ii = TRUE;
        else
            i_ii++;

    if (found_ii)
        return (&mod_net_mod_arm [i_ii]);
    else
        return (0);
    }


static void mod_delete_module (void)
    {
    item_t    items_ari [20];
    module_t *modules_apri [20];
    int       i_ii;
    int       nr_items_ii = 0;
    char      buffer_ti [MAX_X];
    FILE     *fd_pri;
    int       choice_ii;


    fd_pri = fopen ("/proc/modules", "r");
    if (!fd_pri)
        return;

    while (fgets (buffer_ti, MAX_X - 1, fd_pri) &&
           nr_items_ii < sizeof (items_ari) / sizeof (items_ari [0]))
        {
        i_ii = 0;
        while (buffer_ti [i_ii] != ' ')
            i_ii++;
        buffer_ti [i_ii] = 0;
        modules_apri [nr_items_ii] = mod_get_description (buffer_ti);
        if (modules_apri [nr_items_ii])
            {
            items_ari [nr_items_ii].text = malloc (MENU_WIDTH);
            items_ari [nr_items_ii].func = 0;
            strncpy (items_ari [nr_items_ii].text,
                     modules_apri [nr_items_ii]->description, MENU_WIDTH);
            util_fill_string (items_ari [nr_items_ii].text, MENU_WIDTH);
            nr_items_ii++;
            }
        }
    fclose (fd_pri);

    if (!nr_items_ii)
        (void) dia_message (txt_get (TXT_NO_MODULES), MSGTYPE_INFO);
    else
        {
        choice_ii = dia_menu (txt_get (TXT_DELETE_MODULE), items_ari, nr_items_ii, 1);
        if (choice_ii)
            {
            if (mod_is_ppcd (modules_apri [choice_ii - 1]->module_name))
                {
                mod_unload_module ("pcd");
                mpar_delete_modparams ("pcd");
                }

            mod_unload_module (modules_apri [choice_ii - 1]->module_name);
            mpar_delete_modparams (modules_apri [choice_ii - 1]->module_name);
            }

        for (i_ii = 0; i_ii < nr_items_ii; i_ii++)
            free (items_ari [i_ii].text);
        }
    }


static int mod_menu_cb (int what_iv)
    {
           int   rc_ii;
           char  text_ti [200];
    static int   warned_is = FALSE;


    switch (what_iv)
        {
        case 1:
            if (!warned_is && info_scsi_exists ())
                {
                sprintf (text_ti, txt_get (TXT_ALREADY_FOUND),
                                  txt_get (TXT_SCSI_ADAPTER));
                strcat (text_ti, txt_get (TXT_ANOTHER_MOD));
                rc_ii = dia_yesno (text_ti, 1);
                warned_is = TRUE;
                }
            else
                rc_ii = YES;

            if (rc_ii == YES)
                {
                mod_show_kernel_im = TRUE;
                rc_ii = mod_load_by_user (MOD_TYPE_SCSI);
                if (rc_ii && !scsi_tg [0])
                    {
                    strcpy (scsi_tg, mod_current_arm [rc_ii - 1].module_name);
                    strcpy (cdrom_tg, "scd0");
                    }
                mod_show_kernel_im = FALSE;
                }
            break;
        case 2:
            if (!warned_is && (info_scsi_cd_exists () || info_eide_cd_exists ()))
                {
                sprintf (text_ti, txt_get (TXT_ALREADY_FOUND),
                                  txt_get (TXT_CDROM));
                strcat (text_ti, txt_get (TXT_ANOTHER_MOD));
                rc_ii = dia_yesno (text_ti, 1);
                warned_is = TRUE;
                }
            else
                rc_ii = YES;

            if (rc_ii == YES)
                {
                rc_ii = mod_load_by_user (MOD_TYPE_OTHER);
                if (rc_ii && !mod_is_ppcd (mod_current_arm [rc_ii - 1].module_name))
                    strcpy (cdrom_tg, mod_current_arm [rc_ii - 1].module_name);
                }
            break;
        case 3:
            rc_ii = mod_load_by_user (MOD_TYPE_NET);
            if (rc_ii && !net_tg [0])
                strcpy (net_tg, mod_current_arm [rc_ii - 1].module_name);
            break;
        case 4:
            (void) pcmcia_load_core ();
            break;
        case 5:
            mod_show_modules ();
            break;
        case 6:
            mod_delete_module ();
            break;
        case 7:
            mod_autoload ();
            break;
        default:
            break;
        }

    return (what_iv);
    }


static void mod_load_core (void)
    {
    static int   core_loaded_is = FALSE;
           char *core_modules_ati [] = {
                                       "8390",
                                       "cdrom"
                                       };
           int   i_ii;
           char  tmp_ti [100];


    if (!core_loaded_is)
        {
        for (i_ii = 0;
             i_ii < sizeof (core_modules_ati) / sizeof (core_modules_ati [0]);
             i_ii++)
            {
            mod_load_module (core_modules_ati [i_ii], 0);
            sprintf (tmp_ti, "%s/%s.o", mod_modpath_tm, core_modules_ati [i_ii]);
            unlink (tmp_ti);
            }

        core_loaded_is = TRUE;
        }
    }


static int mod_get_current_list (int mod_type_iv, int *nr_modules_pir,
                                 int *more_pir)
    {
    module_t      *test_modules_pri;
    struct dirent *module_pri;
    DIR           *directory_ri;
    int            i_ii;
    int            max_ii = 0;
    int            found_ii;
    char           tmp_ti [30];


    switch (mod_type_iv)
        {
        case MOD_TYPE_SCSI:
            test_modules_pri = mod_scsi_mod_arm;
            max_ii = NR_SCSI_MODULES;
            break;
        case MOD_TYPE_OTHER:
            test_modules_pri = mod_cdrom_mod_arm;
            max_ii = NR_CDROM_MODULES;
            break;
        case MOD_TYPE_NET:
            test_modules_pri = mod_net_mod_arm;
            max_ii = NR_NET_MODULES;
            break;
        default:
            return (-1);
            break;
        }

    directory_ri = opendir (mod_modpath_tm);
    if (!directory_ri)
        return (-1);

    *nr_modules_pir = 0;
    *more_pir = FALSE;
    module_pri = readdir (directory_ri);
    while (module_pri)
        {
        i_ii = 0;
        found_ii = FALSE;

        while (i_ii < max_ii && !found_ii)
            {
            sprintf (tmp_ti, "%s.o", test_modules_pri [i_ii].module_name);
            if (!strcmp (tmp_ti, module_pri->d_name))
                found_ii = TRUE;
            else
                i_ii++;
            }

        if (found_ii)
            memcpy (&mod_current_arm [(*nr_modules_pir)++], &test_modules_pri [i_ii],
                    sizeof (module_t));
        else if (!strcmp ("MORE", module_pri->d_name))
            *more_pir = TRUE;

        module_pri = readdir (directory_ri);
        }

    (void) closedir (directory_ri);

    return (0);
    }


static int mod_load_ppcd_core (void)
    {
    static int       ppcd_core_loaded_is = FALSE;
           int       rc_ii;
           window_t  status_ri;
           int       i_ii = 0;

    if (!ppcd_core_loaded_is)
        {
        dia_status_on (&status_ri, txt_get (TXT_INIT_PARPORT));
        rc_ii = system ("insmod /modules/lp.o");
        if (rc_ii)
            {
            win_close (&status_ri);
            return (rc_ii);
            }

        while (i_ii < 50)
            {
            dia_status (&status_ri, i_ii++);
            usleep (10000);
            }

        rc_ii = system ("rmmod lp");
        if (rc_ii)
            {
            win_close (&status_ri);
            return (rc_ii);
            }

        while (i_ii <= 100)
            {
            dia_status (&status_ri, i_ii++);
            usleep (10000);
            }

        rc_ii = system ("insmod /modules/paride.o");
        win_close (&status_ri);
        if (rc_ii)
            {
            (void) system ("rmmod lp");
            return (-1);
            }

        ppcd_core_loaded_is = TRUE;
        }

    return (0);
    }
