/*
 *
 * settings.c    Settings for linuxrc
 *
 * Copyright (c) 1996-1998  Hubert Mantel, S.u.S.E. GmbH  (mantel@suse.de)
 *
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "global.h"
#include "text.h"
#include "util.h"
#include "display.h"
#include "keyboard.h"
#include "dialog.h"


typedef struct
      {
      char *descr;
      char *mapname;
      } keymap_t;

static keymap_t set_keymaps_arm [] =
{
{ "Deutsch",   "de-lat1-nd" },
{ "American",  "us"         },
{ "Suomi",     "fi-latin1"  },
{ "British",   "uk"         },
{ "Franais",  "fr-latin1"  },
{ "Espaol",   "es"         },
{ "Italiano",  "it"         }
};

#define NR_KEYMAPS (sizeof(set_keymaps_arm)/sizeof(set_keymaps_arm[0]))


/*
 *
 * local function prototypes
 *
 */

static int  set_settings_cb (int what_iv);
static void set_expert      (void);
static int  set_expert_cb   (int what_iv);

/*
 *
 * exported functions
 *
 */

int set_settings (void)
    {
    item_t items_ari [4];
    int    nr_items_ii = sizeof (items_ari) / sizeof (items_ari [0]);
    int    i_ii;
    int    width_ii = 30;
    int    choice_ii;


    util_create_items (items_ari, nr_items_ii, width_ii);

    strncpy (items_ari [0].text, txt_get (TXT_MENU_LANG), width_ii);
    strncpy (items_ari [1].text, txt_get (TXT_MENU_DISPLAY), width_ii);
    strncpy (items_ari [2].text, txt_get (TXT_MENU_KEYMAP), width_ii);
    strncpy (items_ari [3].text, txt_get (TXT_MENU_EXPERT), width_ii);
    for (i_ii = 0; i_ii < nr_items_ii; i_ii++)
        {
        util_center_text (items_ari [i_ii].text, width_ii);
        items_ari [i_ii].func = set_settings_cb;
        }

    choice_ii =  dia_menu (txt_get (TXT_SETTINGS), items_ari,
                           nr_items_ii, 1);

    util_free_items (items_ari, nr_items_ii);

    return (choice_ii);
    }


void set_choose_display (void)
    {
    item_t   items_ari [2];
    int      choice_ii;
    int      width_ii = 30;


    if (auto_ig)
        {
        disp_set_display (color_ig ? 1 : 2);
        return;
        }

    util_create_items (items_ari, 2, width_ii);
    strcpy (items_ari [0].text, txt_get (TXT_COLOR_DISPLAY));
    strcpy (items_ari [1].text, txt_get (TXT_MONO_DISPLAY));
    util_center_text (items_ari [0].text, width_ii);
    util_center_text (items_ari [1].text, width_ii);

    choice_ii = dia_menu (txt_get (TXT_CHOOSE_DISPLAY), items_ari, 2, 1);
    disp_set_display (choice_ii);

    util_free_items (items_ari, 2);
    }


void set_choose_keytable (void)
    {
    item_t  items_ari [NR_KEYMAPS];
    int     i_ii;
    int     width_ii = 20;
    char    command_ti [MAX_FILENAME];


    if (auto_ig)
        {
        if (!keymap_tg [0])
            strcpy (keymap_tg, set_keymaps_arm [0].mapname);

        sprintf (command_ti, "loadkeys %s.map", keymap_tg);
        system (command_ti);
        return;
        }

    util_create_items (items_ari, NR_KEYMAPS, width_ii);
    for (i_ii = 0; i_ii < NR_KEYMAPS; i_ii++)
        {
        strcpy (items_ari [i_ii].text, set_keymaps_arm [i_ii].descr);
        util_center_text (items_ari [i_ii].text, width_ii);
        }

    i_ii = dia_menu (txt_get (TXT_CHOOSE_KEYMAP), items_ari, NR_KEYMAPS, 1);
    util_free_items (items_ari, NR_KEYMAPS);

    if (i_ii)
        {
        sprintf (command_ti, "loadkeys %s.map",
                 set_keymaps_arm [i_ii - 1].mapname);
        system (command_ti);
        strcpy (keymap_tg, set_keymaps_arm [i_ii - 1].mapname);
        }
    }


void set_choose_language (void)
    {
    item_t  items_ari [4];
    int     nr_items_ii = sizeof (items_ari) / sizeof (items_ari [0]);
    int     current_ii;
    int     rc_ii;
    int     width_ii = 20;
    int     i_ii;


    if (auto_ig)
        {
        if (language_ig == LANG_UNDEF)
            language_ig = LANG_GERMAN;
        return;
        }

    switch (language_ig)
        {
        case LANG_ENGLISH:
            current_ii = 2;
            break;
        case LANG_ITALIAN:
            current_ii = 3;
            break;
        case LANG_FRENCH:
            current_ii = 4;
            break;
        default:
            current_ii = 1;
            break;
        }

    util_create_items (items_ari, nr_items_ii, width_ii);
    strcpy (items_ari [0].text, "Deutsch");
    strcpy (items_ari [1].text, "English");
    strcpy (items_ari [2].text, "Italiano");
    strcpy (items_ari [3].text, "Franais");

    for (i_ii = 0; i_ii < nr_items_ii; i_ii++)
        util_center_text (items_ari [i_ii].text, width_ii);

    rc_ii = dia_menu (txt_get (TXT_CHOOSE_LANGUAGE), items_ari, nr_items_ii,
                      current_ii);

    switch (rc_ii)
        {
        case 1:
            language_ig = LANG_GERMAN;
            break;
        case 2:
            language_ig = LANG_ENGLISH;
            break;
        case 3:
            language_ig = LANG_ITALIAN;
            break;
        case 4:
            language_ig = LANG_FRENCH;
            break;
        default:
            break;
        }

    util_free_items (items_ari, nr_items_ii);
    }


static int set_settings_cb (int what_iv)
    {
    int  rc_ii = 0;

    switch (what_iv)
        {
        case 1:
            set_choose_language ();
            break;
        case 2:
            set_choose_display ();
            util_print_banner ();
            break;
        case 3:
            set_choose_keytable ();
            rc_ii = 1;
            break;
        case 4:
            set_expert ();
            rc_ii = 1;
            break;
        default:
            break;
        }

    return (rc_ii);
    }


static void set_expert (void)
    {
    item_t     items_ari [5];
    int        nr_items_ii = sizeof (items_ari) / sizeof (items_ari [0]);
    int        i_ii;
    int        width_ii = 32;
    char       tmp_ti [MAX_X];
    time_t     time_ri;
    struct tm *broken_time_ri;


    util_create_items (items_ari, nr_items_ii, width_ii);

    strncpy (items_ari [0].text, txt_get (TXT_ASK_ANIMATE), width_ii);
    strncpy (items_ari [1].text, txt_get (TXT_FORCE_ROOTIMAGE), width_ii);
    strncpy (items_ari [2].text, txt_get (TXT_NEW_ROOTIMAGE), width_ii);
    strncpy (items_ari [3].text, txt_get (TXT_NEW_INST_SYS), width_ii);
    strncpy (items_ari [4].text, txt_get (TXT_NFSPORT), width_ii);
    for (i_ii = 0; i_ii < nr_items_ii; i_ii++)
        {
        util_center_text (items_ari [i_ii].text, width_ii);
        items_ari [i_ii].func = set_expert_cb;
        }

    time_ri = time (0);
    broken_time_ri = gmtime (&time_ri);
    sprintf (tmp_ti, "%s -- Time: %02d:%02d", txt_get (TXT_MENU_EXPERT),
             broken_time_ri->tm_hour, broken_time_ri->tm_min);
    (void) dia_menu (tmp_ti, items_ari, nr_items_ii, 1);

    util_free_items (items_ari, nr_items_ii);
    }


static int set_expert_cb (int what_iv)
    {
    char  tmp_ti [MAX_FILENAME];
    int   rc_ii;


    switch (what_iv)
        {
        case 1:
            rc_ii = dia_yesno (txt_get (TXT_ASK_EXPLODE),
                               explode_win_ig == TRUE ? YES : NO);
            if (rc_ii == YES)
                explode_win_ig = TRUE;
            else if (rc_ii == NO)
                explode_win_ig = FALSE;
            break;
        case 2:
            rc_ii = dia_yesno (txt_get (TXT_ASK_RI_FORCE),
                               force_ri_ig == TRUE ? YES : NO);
            if (rc_ii == YES)
                force_ri_ig = TRUE;
            else if (rc_ii == NO)
                force_ri_ig = FALSE;
            break;
        case 3:
            strcpy (tmp_ti, rootimage_tg);
            rc_ii = dia_input (txt_get (TXT_ENTER_ROOTIMAGE), tmp_ti,
                               MAX_FILENAME - 1, 30);
            if (!rc_ii)
                strcpy (rootimage_tg, tmp_ti);
            break;
        case 4:
            strcpy (tmp_ti, installdir_tg);
            rc_ii = dia_input (txt_get (TXT_ENTER_INST_SYS), tmp_ti,
                               MAX_FILENAME - 1, 30);
            if (!rc_ii)
                strcpy (installdir_tg, tmp_ti);
            break;
        case 5:
            if (nfsport_ig)
                sprintf (tmp_ti, "%d", nfsport_ig);
            else
                tmp_ti [0] = 0;
            rc_ii = dia_input (txt_get (TXT_ENTER_NFSPORT), tmp_ti, 6, 6);
            if (!rc_ii)
                nfsport_ig = atoi (tmp_ti);
            break;
        default:
            dia_message (txt_get (TXT_NOT_IMPLEMENTED), MSGTYPE_ERROR);
            break;
        }

    return (what_iv);
    }
