/*
 *
 * text.c        Handling of messages
 *
 * Copyright (c) 1996-1998  Hubert Mantel, S.u.S.E. GmbH  (mantel@suse.de)
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "global.h"
#include "text.h"

typedef struct
    {
    enum textid_t  id;
    char          *text;
    } text_t;

static text_t txt_german_atm [] =
{
{ TXT_OK,             "Ok"                                                    },
{ TXT_CANCEL,         "Zurck"                                                },
{ TXT_YES,            "Ja"                                                    },
{ TXT_NO,             "Nein"                                                  },
{ TXT_CHOOSE_LANGUAGE,"Bitte whlen Sie die Sprache"                          },
{ TXT_COLOR_DISPLAY,  "Farbbildschirm"                                        },
{ TXT_MONO_DISPLAY,   "Monochrom-Bildschirm"                                  },
{ TXT_CHOOSE_DISPLAY, "Welche Art von Bildschirm verwenden Sie?"              },
{ TXT_MENU_LANG,      "Sprache"                                               },
{ TXT_MENU_MODULES,   "Kernel-Module (Hardware-Treiber)"                      },
{ TXT_MENU_START,     "Installation / System starten"                         },
{ TXT_MENU_INFO,      "System-Information"                                    },
{ TXT_HDR_MAIN,       "Hauptmen"                                             },
{ TXT_SETTINGS,       "Einstellungen"                                         },
{ TXT_MENU_DISPLAY,   "Bildschirm"                                            },
{ TXT_MENU_KEYMAP,    "Tastaturbelegung"                                      },
{ TXT_LOAD_SCSI,      "SCSI-Treiber laden"                                    },
{ TXT_LOAD_CDROM,     "CD-ROM-Treiber laden"                                  },
{ TXT_LOAD_NET,       "Netzwerktreiber laden"                                 },
{ TXT_SHOW_MODULES,   "Zeige geladene Module"                                 },
{ TXT_AUTO_LOAD,      "Automatisches Laden von Modulen"                       },
{ TXT_DEL_MODULES,    "Entfernen geladener Module"                            },
{ TXT_TRY_TO_LOAD,    "Versuche, das Modul \"%s\" zu laden...\n\n"
                      "Whrend des Ladens knnen Sie die Kernelmeldungen auf "
                      "Konsole 4 (ALT-F4) beobachten."                        },
{ TXT_LOAD_SUCCESSFUL,"Das Modul \"%s\" wurde erfolgreich geladen!"           },
{ TXT_LOAD_FAILED,    "Das Laden des Moduls \"%s\" ist fehlgeschlagen!"       },
{ TXT_CDROM,          "CD-ROM"                                                },
{ TXT_FLOPPY,         "Diskette"                                              },
{ TXT_NFS,            "Netzwerk (NFS)"                                        },
{ TXT_FTP,            "Netzwerk (FTP)"                                        },
{ TXT_HARDDISK,       "Festplatte"                                            },
{ TXT_START_INSTALL,  "Installation starten"                                  },
{ TXT_START_RESCUE,   "Rettungssystem starten"                                },
{ TXT_LOAD_RS_NFS,    "Rettungssystem via NFS laden"                          },
{ TXT_BOOT_SYSTEM,    "Installiertes System booten"                           },
{ TXT_SEARCH_FOR_RI,  "Suche nach Image \"%s\"..."                            },
{ TXT_RI_NOT_FOUND,   "Image konnte nicht gefunden werden!\n\n"
                      "berprfen Sie die Angabe des Verzeichnisses."         },
{ TXT_LOADING,        "Lade Daten in die Ramdisk"                             },
{ TXT_INFO_KERNEL,    "Kernel-Meldungen"                                      },
{ TXT_INFO_CPU,       "Prozessor"                                             },
{ TXT_INFO_DEVICES,   "Gerte"                                                },
{ TXT_INFO_NETDEV,    "Netzwerkkarten"                                        },
{ TXT_INFO_DMA,       "DMA"                                                   },
{ TXT_INFO_IRQS,      "Interrupts"                                            },
{ TXT_INFO_IOPORTS,   "I/O Ports"                                             },
{ TXT_INFO_MEM,       "Speicher"                                              },
{ TXT_INFO_MODULES,   "Module"                                                },
{ TXT_INFO_PCI,       "PCI"                                                   },
{ TXT_INSERT_DISK,    "Bitte legen Sie die Diskette ein\n"
                      "und drcken Sie ENTER"                                 },
{ TXT_ERROR_READ_DISK,"Fehler beim Lesen von der Diskette!"                   },
{ TXT_ENTER_ROOT_FS,  "Bitte geben Sie Ihre Root-Partition an\n"
                      "(z.B. /dev/sda3)"                                      },
{ TXT_INVALID_ROOT_FS,"Ungltige Root-Partition!"                             },
{ TXT_INVALID_INPUT,  "Ungltige Eingabe!"                                    },
{ TXT_INPUT_IPADDR,   "Geben Sie bitte Ihre IP-Adresse ein."                  },
{ TXT_INPUT_NETMASK,  "Geben Sie bitte Ihre Netzmaske ein. Fr ein normales "
                      "Class-C-Netzwerk lautet sie 255.255.255.0"             },
{ TXT_INPUT_GATEWAY,  "Geben Sie die Adresse Ihres Gateways ein, sofern Sie "
                      "eines bentigen, um den Server ansprechen zu knnen. "
                      "Ansonsten geben Sie einfach Ihre eigene IP-Adresse an."},
{ TXT_INPUT_NAMED,    "Geben Sie die IP-Adresse Ihres Nameservers ein. Wenn "
                      "Sie keinen Nameserver verwenden mchten, drcken Sie "
                      "ESC."                                                  },
{ TXT_FOUND_HOST,     "Folgende Rechnerdaten wurden ermittelt:\n\n"
                      "IP-Adresse: %s\n"
                      "Name: %s\n"
                      "Domain: %s"                                            },
{ TXT_ERROR_CONF_NET, "Beim Konfigurieren des Netzwerks ist ein Fehler "
                      "aufgetreten. Entweder Sie haben kein passendes Modul "
                      "geladen oder Ihre Netzwerkkarte wurde "
                      "nicht korrekt vom Kernel erkannt!"                     },
{ TXT_INPUT_SERVER,   "Geben Sie bitte die IP-Adresse des NFS-Servers ein."   },
{ TXT_INPUT_FTPSERVER,"Geben Sie bitte die IP-Adresse des FTP-Servers ein."   },
{ TXT_INPUT_DIR,      "Geben Sie das Verzeichnis auf dem Server ein."         },
{ TXT_ENTER_PARAMS,   "Bitte geben Sie etwaige Parameter fr \"%s\" ein."     },
{ TXT_TRY_NFS_MOUNT,  "Versuche, das Verzeichnis\n\n\"%s:%s\"\n\n zu "
                      "mounten..."                                            },
{ TXT_ERROR_NFSMOUNT, "Beim Versuch, das NFS-Verzeichnis des Servers zu "
                      "mounten, ist folgender Fehler aufgetreten:\n\n"
                      "Fehler vom NFS-Server: %s"                             },
{ TXT_CHOOSE_KEYMAP,  "Bitte whlen Sie eine Tastaturbelegung aus.\nYaST "
                      "bietet Ihnen spter weitere Belegungen an."            },
{ TXT_DELETE_MODULE,  "Whlen Sie das Modul aus, das Sie entfernen mchten."  },
{ TXT_NO_MODULES,     "Momentan sind keine Module geladen!"                   },
{ TXT_AUTO_RUNNING,   "Autoprobing (%s)...      "                             },
{ TXT_WANT_MORE_SCSI, "Mchten Sie nach weiteren SCSI-Adaptern suchen?"       },
{ TXT_WANT_MORE_NET,  "Mchten Sie nach weiteren Netzwerkkarten suchen?"      },
{ TXT_WANT_MORE_CDROM,"Mchten Sie nach weiteren CD-ROM-Laufwerken suchen?"   },
{ TXT_NET_ETH0,       "Ethernet Netzwerkkarte"                                },
{ TXT_NET_PLIP,       "Parallelport"                                          },
{ TXT_NET_TR0,        "Token Ring Netzwerkkarte"                              },
{ TXT_NET_ARC0,       "Arcnet Netzwerkkarte"                                  },
{ TXT_CHOOSE_NET,     "Bitte whlen Sie den Typ Netzwerk, den Sie fr die "
                      "Installation verwenden mchten."                       },
{ TXT_INPUT_PLIP_IP,  "Geben Sie bitte die IP-Adresse des PLIP-Partners ein." },
{ TXT_LOAD_PCMCIA,    "Lade PCMCIA Module"                                    },
{ TXT_NO_PCMCIA,      "Es wurde keine PCMCIA Hardware gefunden!"              },
{ TXT_FOUND_PCMCIA,   "PCMCIA Chipsatz \"%s\" gefunden!\n\nBitte geben Sie "
                      "etwaige Parameter fr die PCMCIA Core-Module ein"      },
{ TXT_PCMCIA_PARAMS,  "Bitte geben Sie etwaige spezifische Parameter fr den "
                      "Chipsatz \"%s\" ein"                                   },
{ TXT_START_CARDMGR,  "Cardmanager wird gestartet..."                         },
{ TXT_PCMCIA_SUCCESS, "PCMCIA Card Services wurden erfolgreich gestartet."    },
{ TXT_PCMCIA_FAIL,    "Das Starten der PCMCIA Card Services ist gescheitert!" },
{ TXT_PCMCIA_ALREADY, "PCMCIA Card Services laufen bereits!"                  },
{ TXT_NO_LXRC_END,    "Linuxrc kann nur ber den Menpunkt\n\n"                 
                      " Installation / System starten \n\n"
                      "verlassen werden! Damit dies mglich ist, mssen Sie "
                      "eventuell vorher Hardware-Treiber (Module) laden."     },
{ TXT_NO_CTRLC,       "Linuxrc kann nicht durch <ctrl-c> abgebrochen werden!" },
{ TXT_REMOVE_DISK,    "Bitte entfernen Sie nun die Diskette aus dem Laufwerk!"},
{ TXT_EXAMPLE_PAR,    "\n\nBeispiel: "                                        },
{ TXT_CHOOSE_SOURCE,  "Whlen Sie bitte das Quellmedium"                      },
{ TXT_TRY_CD_MOUNT,   "Versuche, die CD-ROM zu mounten..."                    },
{ TXT_ERROR_CD_MOUNT, "CD-ROM konnte nicht gemountet werden!"                 },
{ TXT_START_YAST,     "Starte YaST..."                                        },
{ TXT_ASK_REBOOT,     "Mchten Sie das System nun neu starten?"               },
{ TXT_INSTALL_SUCCESS,"Das Basissystem wurde erfolgreich installiert. Dieses "
                      "neu installierte System wird nun gestartet, um die "
                      "Installation abzuschlieen."                           },
{ TXT_ENTER_PARTITION,"Bitte geben Sie die Festplatten-Partition an\n"
                      "(z.B. /dev/sda1)"                                      },
{ TXT_ERROR_HD_MOUNT, "Die angegebene Partition konnte nicht gemountet "
                      "werden!"                                               },
{ TXT_ENTER_HD_DIR,   "Bitte geben Sie das Quellverzeichnis ein"              },
{ TXT_ERROR_INSTALL,  "Die Installation wurde nicht ordnungsgem "
                      "abgeschlossen!"                                        },
{ TXT_DRIVES,         "Festplatten / CD-ROMs"                                 },
{ TXT_LITTLE_MEM,     "Ihr Rechner verfgt ber extrem wenig Hauptspeicher!\n"
                      "\nBevor Sie diese Meldung besttigen und YaST gestartet "
                      "werden kann, mssen Sie auf Konsole 2 wechseln "
                      "(drcken Sie ALT-F2) und von Hand Swap-Speicher "
                      "einrichten und aktivieren!"                            },
{ TXT_ASK_ANIMATE,    "Animation"                                             },
{ TXT_ASK_EXPLODE,    "Mchten Sie \"explodierende\" Fenster?\n\n"
                      "Aber klar ;-)"                                         },
{ TXT_NO_DRIVES,      "Bislang wurden weder Festplatten noch CD-ROM Laufwerke"
                      " erkannt."                                             },
{ TXT_MENU_EXPERT,    "Debug (Experte)"                                       },
{ TXT_FORCE_ROOTIMAGE,"Erzwinge Rootimage"                                    },
{ TXT_NEW_ROOTIMAGE,  "Eingabe des Rootimage"                                 },
{ TXT_ASK_RI_FORCE,   "Soll das Rootimage auch dann in die RAM-Disk geladen "
                      "werden, wenn ein gltiges Installationssystem "
                      "gefunden wurde?"                                       },
{ TXT_ENTER_ROOTIMAGE,"Geben Sie Pfad und Namen der Datei ein, die Sie als "
                      "Root Filesystem in die RAM-Disk laden mchten."        },
{ TXT_NOT_IMPLEMENTED,"Noch nicht implementiert!"                             },
{ TXT_NEW_INST_SYS,   "Installationssystem"                                   },
{ TXT_ENTER_INST_SYS, "Geben Sie das Verzeichnis ein, das das "
                      "Installationssystem enthlt."                          },
{ TXT_NO_INFO_AVAIL,  "Keine Informationen verfgbar."                        },
{ TXT_PREPARE_INST,   "Installation wird vorbereitet..."                      },
{ TXT_START_DEMO,     "Live-CD starten"                                       },
{ TXT_INSERT_LIVECD,  "Bitte legen Sie die Live-CD ein."                      },
{ TXT_END_REBOOT,     "Abbruch / Reboot"                                      },
{ TXT_ASK_BOOTP,      "Automatische Konfiguration via Bootp?"                 },
{ TXT_SEND_BOOTP,     "Bootp-Anfrage wird gesendet..."                        },
{ TXT_ERROR_BOOTP,    "Bootp-Konfiguration ist gescheitert."                  },
{ TXT_SEARCH_INFOFILE,"Suche nach Info-Datei..."                              },
{ TXT_NFSPORT,        "Angabe des NFS-Ports"                                  },
{ TXT_ASKING_NAMED,   "Befrage Nameserver..."                                 },
{ TXT_HOST_NOT_FOUND, "Rechner %s ist unbekannt!"                             },
{ TXT_NET_CONFIGURED, "Ihr Netzwerk ist bereits konfiguriert. Mchten Sie "
                      "die Konfiguration beibehalten?"                        },
{ TXT_ASK_BREAK,      "Mchten Sie abbrechen?"                                },
{ TXT_NO_NETDEVICE,   "Es wurde keine Netzwerkkarte gefunden!\nLaden Sie bitte "
                      "ein entsprechendes Modul."                             },
{ TXT_ALREADY_FOUND,  "In Ihrem System wurde bereits ein %s gefunden!\n"      },
{ TXT_SCSI_ADAPTER,   "SCSI-Adapter"                                          },
{ TXT_ANOTHER_MOD,    "Mchten Sie ein weiteres Modul laden?"                 },
{ TXT_ERROR_FTP,      "Fehler beim Zugriff auf den FTP-Server:\n\n%s"         },
{ TXT_TRY_REACH_FTP,  "Versuche, den FTP-Server zu erreichen..."              },
{ TXT_ENTER_FTPUSER,  "Bitte geben Sie den Benutzernamen ein, unter dem Sie "
                      "auf den FTP-Server zugreifen mchten"                  },
{ TXT_ENTER_FTPPASSWD,"Bitte geben Sie Ihr Passwort auf dem FTP-Server ein"   },
{ TXT_ENTER_FTPPROXY, "Bitte geben Sie die Adresse des FTP Proxy ein"         },
{ TXT_ENTER_FTPPORT,  "Bitte geben Sie den Port des FTP Proxy ein"            },
{ TXT_WANT_FTPPROXY,  "Mchten Sie einen FTP Proxy verwenden?"                },
{ TXT_NOMEM_FTP,      "Fr eine Installation via FTP hat Ihr Rechner nicht "
                      "gengend Hauptspeicher! (mind. 16 MB erforderlich)"    },
{ TXT_ANONYM_FTP,     "Normalerweise erfolgt eine FTP-Installation via "
                      "anonymem FTP. Sie haben jedoch auch die Mglichkeit, "
                      "einen Benutzernamen und ein Passwort anzugeben.\n"
                      "Mchten Sie dies tun?"                                 },
{ TXT_ENTER_MODDISK,  "Bitte legen Sie die Modul-Diskette ein"                },
{ TXT_MORE_MODULES,   "-- Weitere Module --"                                  },
{ TXT_INIT_PARPORT,   "Initialisiere den Parallel-Port..."                    },
{ TXT_ENTER_NFSPORT,  "Bitte geben Sie die Nummer des Ports ein, auf dem der "
                      "NFS-Server luft"                                      }
};

static text_t txt_english_atm [] =
{
{ TXT_OK,             "Ok"                                                    },
{ TXT_CANCEL,         "Back"                                                  },
{ TXT_YES,            "Yes"                                                   },
{ TXT_NO,             "No"                                                    },
{ TXT_CHOOSE_LANGUAGE,"Please choose the language"                            },
{ TXT_COLOR_DISPLAY,  "Color display"                                         },
{ TXT_MONO_DISPLAY,   "Monochrom display"                                     },
{ TXT_CHOOSE_DISPLAY, "What kind of display do you use?"                      },
{ TXT_MENU_LANG,      "Language"                                              },
{ TXT_MENU_MODULES,   "Kernel modules (hardware drivers)"                     },
{ TXT_MENU_START,     "Start installation / system"                           },
{ TXT_MENU_INFO,      "System information"                                    },
{ TXT_HDR_MAIN,       "Main menu"                                             },
{ TXT_SETTINGS,       "Settings"                                              },
{ TXT_MENU_DISPLAY,   "Display"                                               },
{ TXT_MENU_KEYMAP,    "Keymap"                                                },
{ TXT_LOAD_SCSI,      "Load SCSI module"                                      },
{ TXT_LOAD_CDROM,     "Load CD-ROM module"                                    },
{ TXT_LOAD_NET,       "Load network card module"                              },
{ TXT_SHOW_MODULES,   "Show loaded modules"                                   },
{ TXT_AUTO_LOAD,      "Autoload of modules"                                   },
{ TXT_DEL_MODULES,    "Unload modules"                                        },
{ TXT_TRY_TO_LOAD,    "Trying to load module \"%s\"...\n\n"
                      "During loading you may want to watch the kernel "
                      "messages on virtual console 4 (ALT-F4)."               },
{ TXT_LOAD_SUCCESSFUL,"Module \"%s\" has been successfully loaded!"           },
{ TXT_LOAD_FAILED,    "Failed to load module \"%s\"!"                         },
{ TXT_CDROM,          "CD-ROM"                                                },
{ TXT_FLOPPY,         "Floppy"                                                },
{ TXT_NFS,            "Network (NFS)"                                         },
{ TXT_FTP,            "Network (FTP)"                                         },
{ TXT_HARDDISK,       "Harddisk"                                              },
{ TXT_START_INSTALL,  "Start installation"                                    },
{ TXT_START_RESCUE,   "Start rescue system"                                   },
{ TXT_LOAD_RS_NFS,    "Load rescue system via NFS"                            },
{ TXT_BOOT_SYSTEM,    "Boot installed system"                                 },
{ TXT_SEARCH_FOR_RI,  "Searching for image \"%s\"..."                         },
{ TXT_RI_NOT_FOUND,   "Cannot find the image!\n\n"
                      "Check the directory specification!"                    },
{ TXT_LOADING,        "Loading data into ramdisk"                             },
{ TXT_INFO_KERNEL,    "Kernel messages"                                       },
{ TXT_INFO_CPU,       "Processor"                                             },
{ TXT_INFO_DEVICES,   "Devices"                                               },
{ TXT_INFO_NETDEV,    "Network cards"                                         },
{ TXT_INFO_DMA,       "DMA"                                                   },
{ TXT_INFO_IRQS,      "Interrupts"                                            },
{ TXT_INFO_IOPORTS,   "I/O Ports"                                             },
{ TXT_INFO_MEM,       "Memory"                                                },
{ TXT_INFO_MODULES,   "Modules"                                               },
{ TXT_INFO_PCI,       "PCI"                                                   },
{ TXT_INSERT_DISK,    "Please insert the disk and press ENTER"                },
{ TXT_ERROR_READ_DISK,"Error while reading disk!"                             },
{ TXT_ENTER_ROOT_FS,  "Please enter your root partition (eg. /dev/sda3)"      },
{ TXT_INVALID_ROOT_FS,"Invalid root device!"                                  },
{ TXT_INVALID_INPUT,  "Invalid input!"                                        },
{ TXT_INPUT_IPADDR,   "Please enter your IP address"                          },
{ TXT_INPUT_NETMASK,  "Please enter your netmask. For a normal class C "
                      "network, this is usually 255.255.255.0"                },
{ TXT_INPUT_GATEWAY,  "If you need a gateway to reach the server, please "
                      "enter the IP address of the gateway. If you do not "
                      "need a gateway, just enter your own IP address."       },
{ TXT_INPUT_NAMED,    "Please enter the IP address of your name server. If "
                      "you don't use a name server, just press ESC."          },
{ TXT_FOUND_HOST,     "The following data have been retrieved:\n\n"
                      "IP address: %s\n"
                      "Name: %s\n"
                      "Domain: %s"                                            },
{ TXT_ERROR_CONF_NET, "An error occurred while configuring the network. "
                      "Probably your network card has not been recognized "
                      "by the kernel!"                                        },
{ TXT_INPUT_SERVER,   "Please enter the IP address of the NFS server."        },
{ TXT_INPUT_FTPSERVER,"Please enter the IP address of the FTP server."        },
{ TXT_INPUT_DIR,      "Please enter the directory on the server."             },
{ TXT_ENTER_PARAMS,   "Please enter parameters for \"%s\"."                   },
{ TXT_TRY_NFS_MOUNT,  "Trying to mount the directory\n\n\"%s:%s\"..."         },
{ TXT_ERROR_NFSMOUNT, "An error occurred while trying to mount the nfs "
                      "directory.\n\nReason given by server: %s"              },
{ TXT_CHOOSE_KEYMAP,  "Please choose a keyboard map.\nYaST will offer "    
                      "you additional keyboard tables later on."              },
{ TXT_DELETE_MODULE,  "Choose the module to be unloaded."                     },
{ TXT_NO_MODULES,     "Currently there are no modules loaded!"                },
{ TXT_AUTO_RUNNING,   "Autoprobing (%s)...      "                             },
{ TXT_WANT_MORE_SCSI, "Do you want to search for additional SCSI host "
                      "adaptors?"                                             },
{ TXT_WANT_MORE_NET,  "Do you want to search for additional network cards?"   },
{ TXT_WANT_MORE_CDROM,"Do you want to search for additional CD-ROM drives?"   },
{ TXT_NET_ETH0,       "ethernet network card"                                 },
{ TXT_NET_PLIP,       "parallel port"                                         },
{ TXT_NET_TR0,        "token ring network card"                               },
{ TXT_NET_ARC0,       "arcnet network card"                                   },
{ TXT_CHOOSE_NET,     "Please choose the network device."                     },
{ TXT_INPUT_PLIP_IP,  "Please enter the IP-address of the PLIP partner."      },
{ TXT_LOAD_PCMCIA,    "Load PCMCIA modules"                                   },
{ TXT_NO_PCMCIA,      "No PCMCIA hardware found!"                             },
{ TXT_FOUND_PCMCIA,   "Found PCMCIA chipset \"%s\"!\n\nPlease enter "
                      "parameters for PCMCIA core modules"                    },
{ TXT_PCMCIA_PARAMS,  "Please enter specific parameters for chipset \"%s\""   },
{ TXT_START_CARDMGR,  "Starting cardmanager..."                               },
{ TXT_PCMCIA_SUCCESS, "Successfully started PCMCIA card services."            },
{ TXT_PCMCIA_FAIL,    "Start of PCMCIA card services failed!"                 },
{ TXT_PCMCIA_ALREADY, "PCMCIA card services already running!"                 },
{ TXT_NO_LXRC_END,    "You can exit Linuxrc only via the menu item\n\n"
                      " Start installation / system \n\n"
                      "In order to allow this, you might need to load some "
                      "drivers (modules) to support your hardware."           },
{ TXT_NO_CTRLC,       "You cannot exit Linuxrc by pressing <ctrl-c>!"         },
{ TXT_REMOVE_DISK,    "Please remove the disk from the drive now!"            },
{ TXT_EXAMPLE_PAR,    "\n\nExample: "                                         },
{ TXT_CHOOSE_SOURCE,  "Please choose the source media"                        },
{ TXT_TRY_CD_MOUNT,   "Trying to mount the CD-ROM..."                         },
{ TXT_ERROR_CD_MOUNT, "Could not mount the CD-ROM!"                           },
{ TXT_START_YAST,     "Starting YaST..."                                      },
{ TXT_ASK_REBOOT,     "Do you want to reboot the system now?"                 },
{ TXT_INSTALL_SUCCESS,"The base system has been successfully installed. This "
                      "new installed system will now be started in order to "
                      "commit the installation."                              },
{ TXT_ENTER_PARTITION,"Please enter the harddisk partition (eg. /dev/sda1)"   },
{ TXT_ERROR_HD_MOUNT, "Could not mount the specified partition!"              },
{ TXT_ENTER_HD_DIR,   "Please enter the source directory"                     },
{ TXT_ERROR_INSTALL,  "An error occurred during installation!"                },
{ TXT_DRIVES,         "Harddisks / CD-ROMs"                                   },
{ TXT_LITTLE_MEM,     "Your computer has extremely little memory!\n\n"
                      "Before you acknowledge this message you must switch to "
                      "virtual console 2 (just press ALT-F2) and create and "
                      "activate some swap space!"                             },
{ TXT_ASK_ANIMATE,    "Animation"                                             },
{ TXT_ASK_EXPLODE,    "Do you like \"exploding\" windows?\n\nOf course ;-)"   },
{ TXT_NO_DRIVES,      "No harddisks and CD-ROM drives have been detected so "
                      "far."                                                  },
{ TXT_MENU_EXPERT,    "Debug (expert)"                                        },
{ TXT_FORCE_ROOTIMAGE,"Force rootimage"                                       },
{ TXT_NEW_ROOTIMAGE,  "Enter rootimage"                                       },
{ TXT_ASK_RI_FORCE,   "Do you want the rootimage to be loaded into the "       
                      "RAM disk even when a valid installation system could "
                      "be found?"                                             },
{ TXT_ENTER_ROOTIMAGE,"Please enter the path and name of the file you want to"
                      " be loaded into the RAM disk as the root file system"  },
{ TXT_NOT_IMPLEMENTED,"Not yet implemented!"                                  },
{ TXT_NEW_INST_SYS,   "Installation system"                                   },
{ TXT_ENTER_INST_SYS, "Please enter the directory that contains the "
                      "installation system."                                  },
{ TXT_NO_INFO_AVAIL,  "No information available."                             },
{ TXT_PREPARE_INST,   "Preparing installation..."                             },
{ TXT_START_DEMO,     "Start Live CD"                                         },
{ TXT_INSERT_LIVECD,  "Please insert the Live CD."                            },
{ TXT_END_REBOOT,     "End / Reboot"                                          },
{ TXT_ASK_BOOTP,      "Automatic configuration via bootp?"                    },
{ TXT_SEND_BOOTP,     "Sending bootp request..."                              },
{ TXT_ERROR_BOOTP,    "Bootp configuration failed."                           },
{ TXT_SEARCH_INFOFILE,"Searching for infofile..."                             },
{ TXT_NFSPORT,        "Specify NFS port"                                      },
{ TXT_ASKING_NAMED,   "Asking nameserver..."                                  },
{ TXT_HOST_NOT_FOUND, "Machine %s is unknown!"                                },
{ TXT_NET_CONFIGURED, "Your network is already configured. Do you want to "
                      "keep this configuration?"                              },
{ TXT_ASK_BREAK,      "Do want to abort?"                                     },
{ TXT_NO_NETDEVICE,   "No network device found!\nPlease load a network module "
                      "first."                                                },
{ TXT_ALREADY_FOUND,  "A %s was already found in your system!\n"              },
{ TXT_SCSI_ADAPTER,   "SCSI-Adapter"                                          },
{ TXT_ANOTHER_MOD,    "Do you want to load an additional module?"             },
{ TXT_ERROR_FTP,      "Error while accessing the FTP server:\n\n%s"           },
{ TXT_TRY_REACH_FTP,  "Trying to connect to the FTP server..."                },
{ TXT_ENTER_FTPUSER,  "Please enter the user name under which you want to "
                      "access the FTP server"                                 },
{ TXT_ENTER_FTPPASSWD,"Please enter the password for the FTP user account"    },
{ TXT_ENTER_FTPPROXY, "Please enter the address of the FTP proxy"             },
{ TXT_ENTER_FTPPORT,  "Please enter the port of the FTP proxy"                },
{ TXT_WANT_FTPPROXY,  "Do you want to use a FTP proxy?"                       },
{ TXT_NOMEM_FTP,      "Your computer does not have enough RAM to do an "
                      "installation via FTP! (16 MB RAM required)"            },
{ TXT_ANONYM_FTP,     "Normally a FTP installation is performed via "
                      "anonymous FTP. But there is the possibility to specify "
                      "a user name and password on the FTP server.\n"
                      "Do you want this?"                                     },
{ TXT_ENTER_MODDISK,  "Please insert the modules disk"                        },
{ TXT_MORE_MODULES,   "-- More modules --"                                    },
{ TXT_INIT_PARPORT,   "Initializing parallel port..."                         },
{ TXT_ENTER_NFSPORT,  "Please enter the port number the nfs server is running "
                      "on"                                                    }
};

static text_t txt_italian_atm [] =
{
{ TXT_OK,             "Ok"                                                    },
{ TXT_CANCEL,         "Indietro"                                              },
{ TXT_YES,            "Si"                                                    },
{ TXT_NO,             "No"                                                    },
{ TXT_CHOOSE_LANGUAGE,"Scegliere la lingua"                                   },
{ TXT_COLOR_DISPLAY,  "Schermo a colori"                                      },
{ TXT_MONO_DISPLAY,   "Schermo monocromo"                                     },
{ TXT_CHOOSE_DISPLAY, "Quale tipo di schermo usa?"                            },
{ TXT_MENU_LANG,      "Lingua"                                                },
{ TXT_MENU_MODULES,   "Modulo-Kernel (Hardware-Driver)"                       },
{ TXT_MENU_START,     "Installazione / iniziare il sistema"                   },
{ TXT_MENU_INFO,      "Sistema-Informazioni"                                  },
{ TXT_HDR_MAIN,       "Menu principale"                                       },
{ TXT_SETTINGS,       "Settaggi"                                              },
{ TXT_MENU_DISPLAY,   "Schermo"                                               },
{ TXT_MENU_KEYMAP,    "Formattazione della tastiera"                          },
{ TXT_LOAD_SCSI,      "Caricare lo SCSI-driver"                               },
{ TXT_LOAD_CDROM,     "Caricare CD-ROM-driver"                                },
{ TXT_LOAD_NET,       "Caricare i driver per la rete"                         },
{ TXT_SHOW_MODULES,   "Mostrare i moduli caricati"                            },
{ TXT_AUTO_LOAD,      "Carica automatica dei Moduli"                          },
{ TXT_DEL_MODULES,    "Rimuovere i moduli caricati"                           },
{ TXT_TRY_TO_LOAD,    "Cerca di caricare il Modulo \"%s\"...\n\n"
                      "Durante il proceso di caricamento potete controllare "
                      "i messaggi del Kernel sulla Consolle 4 (ALT-F4)"       },
{ TXT_LOAD_SUCCESSFUL,"Il modulo \"%s\"  stato caricato con successo!"       },
{ TXT_LOAD_FAILED,    "Il caricamento del modulo \"%s\"  fallito!"           },
{ TXT_CDROM,          "CD-ROM"                                                },
{ TXT_FLOPPY,         "Dischetto"                                             },
{ TXT_NFS,            "Rete (NFS)"                                            },
{ TXT_FTP,            "Rete (FTP)"                                            },
{ TXT_HARDDISK,       "Disk rigido"                                           },
{ TXT_START_INSTALL,  "Iniziare l'installazione"                              },
{ TXT_START_RESCUE,   "Iniziare il sistema del aiuto"                         },
{ TXT_LOAD_RS_NFS,    "Iniziare il sistema via NFS"                           },
{ TXT_BOOT_SYSTEM,    "Fare il boot del sistema installato"                   },
{ TXT_SEARCH_FOR_RI,  "Cerca Image \"%s\"..."                                 },
{ TXT_RI_NOT_FOUND,   "Image non  stato trovato!\n\n"
                      "Controllate le indicazioni del directory."             },
{ TXT_LOADING,        "Carica i dati nel Ramdisk"                             },
{ TXT_INFO_KERNEL,    "Comunicazioni del Kernel"                              },
{ TXT_INFO_CPU,       "Processore"                                            },
{ TXT_INFO_DEVICES,   "Attrezzi"                                              },
{ TXT_INFO_NETDEV,    "Carte rete"                                            },
{ TXT_INFO_DMA,       "DMA"                                                   },
{ TXT_INFO_IRQS,      "Interrupts"                                            },
{ TXT_INFO_IOPORTS,   "I/O Ports"                                             },
{ TXT_INFO_MEM,       "Memoria"                                               },
{ TXT_INFO_MODULES,   "Moduli"                                                },
{ TXT_INFO_PCI,       "PCI"                                                   },
{ TXT_INSERT_DISK,    "Inserite il dischetto\n"
                      "e premete il tasto ENTER"                              },
{ TXT_ERROR_READ_DISK,"Errore alla lettura dal dischetto!"                    },
{ TXT_ENTER_ROOT_FS,  "Indicate la Vostra partizione Root\n"
                      "(p.e. /dev/sda3)"                                      },
{ TXT_INVALID_ROOT_FS,"Partizione Root non valida!"                           },
{ TXT_INVALID_INPUT,  "Indicazione non valida!"                               },
{ TXT_INPUT_IPADDR,   "Indicate il Vostro indirizzo IP."                      },
{ TXT_INPUT_NETMASK,  "Indicate la Vostra maschera-rete. Per una normale "
                      "Class-C-rete  255.255.255.0"                          },
{ TXT_INPUT_GATEWAY,  "Indicate l'indirizzo del Vostro Gateways, nel caso che "
                      "ne serva uno, per poter indirizzare il Server. "
                      "Altrimeni indicate solo il Vostro indirizzo IP."       },
{ TXT_INPUT_NAMED,    "Indicate l'indirizzo IP del Vostro Nameserver. Se non "
                      "volete usare nessun Nameserver, premete solamente ESC."},
{ TXT_FOUND_HOST,     "I seguenti computer-dati sono stati determinati:\n\n"
                      "Indirizzo IP: %s\n"
                      "Nom d'hte: %s\n"
                      "Domain: %s"                                            },
{ TXT_ERROR_CONF_NET, " avvenuto un errore nel configurare la rete. "
                      "O non avete caricato il modulo adatto o "
                      "la Vostra carta-rete non  stata "
                      "riconosciuta correttamente dal Kernel!"                },
{ TXT_INPUT_SERVER,   "Indicate l'indirizzo IP del NFS-Server."               },
{ TXT_INPUT_FTPSERVER,"Indicate l'indirizzo IP del FTP-Server."               },
{ TXT_INPUT_DIR,      "Indicate il directory sull' Server."                   },
{ TXT_ENTER_PARAMS,   "Indicate eventuali parametri per \"%s\"."              },
{ TXT_TRY_NFS_MOUNT,  "Cerca di fare il mount del directory"
                      "\n\n\"%s:%s\""                                         },
{ TXT_ERROR_NFSMOUNT, "Nel fare il mount del directory NFS del Server "
                      " subentrato il seguente errore:\n\n"
                      "Errore dell' NFS-Server: %s"                           },
{ TXT_CHOOSE_KEYMAP,  "Scegliete un settaggio della tastiera.\nYaST "
                      "Vi dar in seguito altri settaggi."                    },
{ TXT_DELETE_MODULE,  "Scegliete il modulo che desiderate eliminare."         },
{ TXT_NO_MODULES,     "Al momento non sono caricati alcuni moduli!"           },
{ TXT_AUTO_RUNNING,   "Autoprobing (%s)...      "                             },
{ TXT_WANT_MORE_SCSI, "Volete cercare altri SCSI-Adapter?"                    },
{ TXT_WANT_MORE_NET,  "Volete cercare altre schede rete?"                     },
{ TXT_WANT_MORE_CDROM,"Volete cercare altri CD-ROM?"                          },
{ TXT_NET_ETH0,       "Scheda rete Ethernet"                                  },
{ TXT_NET_PLIP,       "Parallelport"                                          },
{ TXT_NET_TR0,        "Scheda rete Token Ring"                                },
{ TXT_NET_ARC0,       "Scheda rete Arcnet"                                    },
{ TXT_CHOOSE_NET,     "Scegliete il tipo di rete che volete usare per "
                      "l'installazione."                                      },
{ TXT_INPUT_PLIP_IP,  "Indicate l'indirizzo IP del PLIP-partner."             },
{ TXT_LOAD_PCMCIA,    "Carica moduli PCMCIA"                                  },
{ TXT_NO_PCMCIA,      "Non  stata trovata nessuna Hardware PCMCIA!"          },
{ TXT_FOUND_PCMCIA,   "Trovato PCMCIA Chipset \"%s\"!\n\nIndicate eventuali "
                      "parametri per i moduli Core PCMCIA"                    },
{ TXT_PCMCIA_PARAMS,  "Indicate eventuali parametri specifici per il "
                      "Chipset \"%s\"."                                       },
{ TXT_START_CARDMGR,  "Il Cardmanager viene iniziato..."                      },
{ TXT_PCMCIA_SUCCESS, "PCMCIA Card Services iniziati con successo."           },
{ TXT_PCMCIA_FAIL,    "L'inizio del PCMCIA Card Services  fallito!"          },
{ TXT_PCMCIA_ALREADY, "I PCMCIA Card Services funzionano gi!"                },
{ TXT_NO_LXRC_END,    "Si pu lasciare Linuxrc solo tramite il Menu\n\n"
                      " Installazione / Iniziare il system \n\n"
                      "Affinch ci sia possibile, dovete eventualmente "
                      "caricare prima l' Hardware-Driver (Moduli)."           },
{ TXT_NO_CTRLC,       "Linuxrc non pu venire interrotto tramite <ctrl-c>!"   },
{ TXT_REMOVE_DISK,    "Togliete ora il dischetto!"                            },
{ TXT_EXAMPLE_PAR,    "\n\nEsempio: "                                         },
{ TXT_CHOOSE_SOURCE,  "Scegliete ora il source-medium"                        },
{ TXT_TRY_CD_MOUNT,   "Provate a fare il mount del CD-ROM..."                 },
{ TXT_ERROR_CD_MOUNT, "Il mount del CD-ROM non  stato possibile!"            },
{ TXT_START_YAST,     "Iniziare YaST..."                                      },
{ TXT_ASK_REBOOT,     "Desiderate ora reiniziare il sistema?"                 },
{ TXT_INSTALL_SUCCESS,"Il sistema di base  stato installato con successo. "
                      "Questo nuovo sistema viene ora avviato, per "
                      "chiudere l'installazione."                             },
{ TXT_ENTER_PARTITION,"Indicate la partizione del disk rigido\n"
                      "(p.e. /dev/sda1)"                                      },
{ TXT_ERROR_HD_MOUNT, "Non si  potuto fare il mount della partizione "
                      "indicata!"                                             },
{ TXT_ENTER_HD_DIR,   "Indicate il source-directory"                          },
{ TXT_ERROR_INSTALL,  "L'installazione non  sata terminata "
                      "regolarmente!"                                         },
{ TXT_DRIVES,         "Disk rigidi / CD-ROMs"                                 },
{ TXT_LITTLE_MEM,     "Il Vostro computer ha a disposizione pochissima "
                      "memoria!\n. Prima di confermare questo messaggio e "
                      "iniziare YaST, dovete cambiare sulla Consolle 2 "
                      "(premere ALT-F2) e sistemare e attivare manualmente "
                      "le memorie Swap!"                                      },
{ TXT_ASK_ANIMATE,    "Animazione"                                            },
{ TXT_ASK_EXPLODE,    "Desiderate finestre \"esplosive\"?\n\n"
                      "Certamente ;-)"                                        },
{ TXT_NO_DRIVES,      "Fin'ora non sono stati riconosciuti n disk rigidi "
                      "n CD-ROM."                                            },
{ TXT_MENU_EXPERT,    "Debug (Esperti)"                                       },
{ TXT_FORCE_ROOTIMAGE,"Forzare il Rootimage"                                  },
{ TXT_NEW_ROOTIMAGE,  "Inserire il Rootimage"                                 },
{ TXT_ASK_RI_FORCE,   "Caricare il Rootimage nel RAM-Disk, anche se  stato "
                      "trovato un sistema di installazione "
                      "valido?"                                               },
{ TXT_ENTER_ROOTIMAGE,"Indicate Path e Nome del file, che volete caricare "
                      "come Root Filesystem nel RAM-Disk."                    },
{ TXT_NOT_IMPLEMENTED,"Non ancora implementato!"                              },
{ TXT_NEW_INST_SYS,   "Sistema d'installazione"                               },
{ TXT_ENTER_INST_SYS, "Indicare il directory, che contiene "
                      "il sistema d'installazione."                           },
{ TXT_NO_INFO_AVAIL,  "Nessuna informazione a disposizione."                  },
{ TXT_PREPARE_INST,   "L'installazione viene preparata..."                    },
{ TXT_START_DEMO,     "Cominciare il Live-CD"                                 },
{ TXT_INSERT_LIVECD,  "Inserite il Live-CD."                                  },
{ TXT_END_REBOOT,     "Interruzione / Reboot"                                 },
{ TXT_ASK_BOOTP,      "Configurazione automatica via Bootp?"                  },
{ TXT_SEND_BOOTP,     "Bootp-Domanda viene inviata..."                        },
{ TXT_ERROR_BOOTP,    "Bootp-Configurazione  fallita."                       },
{ TXT_SEARCH_INFOFILE,"Cerca l' Info-File..."                                 },
{ TXT_NFSPORT,        "Indicare il Port NFS"                                  },
{ TXT_ASKING_NAMED,   "Interrogare il Nameserver..."                          },
{ TXT_HOST_NOT_FOUND, "Il computer %s  sconosciuto!"                         },
{ TXT_NET_CONFIGURED, "La Vostra rete  gi configurata. Desiderate "
                      "mantenere la configurazione?"                          },
{ TXT_ASK_BREAK,      "Volete interrompere?"                                  },
{ TXT_NO_NETDEVICE,   "Non  stata trovata una scheda rete!\nCaricate "
                      "eventualmente un modulo corrispondente."               },
{ TXT_ALREADY_FOUND,  "Nel Vostro sistema  gi stato trovato un %s!\n"       },
{ TXT_SCSI_ADAPTER,   "SCSI-adapter"                                          },
{ TXT_ANOTHER_MOD,    "Volete caricare un altro modulo?"                      },
{ TXT_ERROR_FTP,      "Errore durante l'accesso all server FTP:\n\n%s"        },
{ TXT_TRY_REACH_FTP,  "Prova di collegamento con il server FTP..."            },
{ TXT_ENTER_FTPUSER,  "Inserire il nome dell'utente con cui volete accedere "
                      "al server FTP"                                         },
{ TXT_ENTER_FTPPASSWD,"Inserire la password per l'FTP user account"           },
{ TXT_ENTER_FTPPROXY, "Inserire il l'indirizzo dell' FTP proxy"               },
{ TXT_ENTER_FTPPORT,  "Indicate il port dell' FTP Proxy"                      },
{ TXT_WANT_FTPPROXY,  "Desiderate usare un FTP Proxy?"                        },
{ TXT_NOMEM_FTP,      "Il Vostro computer non ha abbastanza RAM per fare "
                      "un'installazione via FTP! (sono necessari 16 MB RAM)"  },
{ TXT_ANONYM_FTP,     "Normalement, une installation FTP est ralise via "
                      "FTP anonyme. Mais il est possible de spcifier un nom "
                      "d'utilisateur et un mot de passe sur le serveur FTP.\n"
                      "Souhaitez-vous le faire?"                              },
{ TXT_ENTER_MODDISK,  "Inserire il dischetto dei moduli"                      },
{ TXT_MORE_MODULES,   "-- Piu moduli --"                                      },
{ TXT_INIT_PARPORT,   "Inizializzare il port parallelo..."                    },
{ TXT_ENTER_NFSPORT,  "Inserite il numero del Port, sul quale funziona "
                      "il Server NFS"                                         }
};

static text_t txt_french_atm [] =
{
{ TXT_OK,             "Ok"                                                    },
{ TXT_CANCEL,         "Retour"                                                },
{ TXT_YES,            "Oui"                                                   },
{ TXT_NO,             "Non"                                                   },
{ TXT_CHOOSE_LANGUAGE,"Choisissez la langue"                                  },
{ TXT_COLOR_DISPLAY,  "Ecran couleur"                                         },
{ TXT_MONO_DISPLAY,   "Ecran monochrome"                                      },
{ TXT_CHOOSE_DISPLAY, "Quel est le type de votre cran?"                      },
{ TXT_MENU_LANG,      "Langue"                                                },
{ TXT_MENU_MODULES,   "Modules du noyau (pilote matriel)"                    },
{ TXT_MENU_START,     "Dmarrer l'installation / le systme"                  },
{ TXT_MENU_INFO,      "Information relative au systme"                       },
{ TXT_HDR_MAIN,       "Menu principal"                                        },
{ TXT_SETTINGS,       "Configuration"                                         },
{ TXT_MENU_DISPLAY,   "Ecran"                                                 },
{ TXT_MENU_KEYMAP,    "Disposition du clavier"                                },
{ TXT_LOAD_SCSI,      "Charger le pilote SCSI"                                },
{ TXT_LOAD_CDROM,     "Charger le pilote de CD-ROM"                           },
{ TXT_LOAD_NET,       "Charger le pilote rseau"                              },
{ TXT_SHOW_MODULES,   "Prsentation des modules chargs"                      },
{ TXT_AUTO_LOAD,      "Chargement automatique des modules"                    },
{ TXT_DEL_MODULES,    "Suppression de modules chargs"                        },
{ TXT_TRY_TO_LOAD,    "Tentative de charger le module \"%s\"...\n\n"
                      "Pendant le chargement, vous pouvez visualiser les "
                      "messages du noyau sur la console 4 (ALT-F4)."          },
{ TXT_LOAD_SUCCESSFUL,"Chargement du module \"%s\" effectu avec succs!"     },
{ TXT_LOAD_FAILED,    "Le chargement du module \"%s\" a chou!"              },
{ TXT_CDROM,          "CD-ROM"                                                },
{ TXT_FLOPPY,         "Disquette"                                             },
{ TXT_NFS,            "Rseau (NFS)"                                          },
{ TXT_FTP,            "Rseau (FTP)"                                          },
{ TXT_HARDDISK,       "Disque dur"                                            },
{ TXT_START_INSTALL,  "Dmarrer l'installation"                               },
{ TXT_START_RESCUE,   "Dmarrer le systme de sauvegarde"                     },
{ TXT_LOAD_RS_NFS,    "Charger le systme de sauvegarde via NFS"              },
{ TXT_BOOT_SYSTEM,    "Amorcer le systme install"                           },
{ TXT_SEARCH_FOR_RI,  "Recherche de l'image \"%s\"..."                        },
{ TXT_RI_NOT_FOUND,   "L'image n'a pas pu tre repre!\n\n"
                      "Vrifiez la spcification du rpertoire."              },
{ TXT_LOADING,        "Chargement des donnes"                                },
{ TXT_INFO_KERNEL,    "Messages du noyau"                                     },
{ TXT_INFO_CPU,       "Processeur"                                            },
{ TXT_INFO_DEVICES,   "Priphriques"                                         },
{ TXT_INFO_NETDEV,    "Cartes rseau"                                         },
{ TXT_INFO_DMA,       "DMA"                                                   },
{ TXT_INFO_IRQS,      "Interruptions"                                         },
{ TXT_INFO_IOPORTS,   "Ports d'E/S"                                           },
{ TXT_INFO_MEM,       "Mmoire"                                               },
{ TXT_INFO_MODULES,   "Modules"                                               },
{ TXT_INFO_PCI,       "PCI"                                                   },
{ TXT_INSERT_DISK,    "Insrez la disquette\n"
                      "et appuyez sur ENTRE"                                 },
{ TXT_ERROR_READ_DISK,"Erreur lors de la lecture de la disquette!"            },
{ TXT_ENTER_ROOT_FS,  "Indiquez votre partition racine (root)\n"
                      "(par exemple /dev/sda3)"                               },
{ TXT_INVALID_ROOT_FS,"Partition racine non valide!"                          },
{ TXT_INVALID_INPUT,  "Entre non valide!"                                    },
{ TXT_INPUT_IPADDR,   "Entrez votre adresse IP."                              },
{ TXT_INPUT_NETMASK,  "Entrez votre masque de rseau. Pour un rseau normal "
                      "classe C, c'est: 255.255.255.0"                        },
{ TXT_INPUT_GATEWAY,  "Entrez l'adresse de votre passerelle s'il vous en "
                      "faut une pour pouvoir accder au serveur. "
                      "Sinon entrez simplement votre propre adresse IP."      },
{ TXT_INPUT_NAMED,    "Entrez l'adresse IP de votre serveur de noms. Si "
                      "vous ne souhaitez pas utiliser de serveur de noms,"
                      "appuyez sur ESC."                                      },
{ TXT_FOUND_HOST,     "Les donnes suivantes ont t trouves:\n\n"
                      "Adresse IP: %s\n"
                      "Name: %s\n"
                      "Domaine: %s"                                           },
{ TXT_ERROR_CONF_NET, "Lors de la configuration du rseau il s'est produit"
                      "une erreur. Ou vous n'avez pas charg de module "
                      "adquat, ou bien votre carte rseau n'a pas t "
                      "reconnue correctement par le noyau!"                   },
{ TXT_INPUT_SERVER,   "Entrez l'adresse IP du serveur NFS."                   },
{ TXT_INPUT_FTPSERVER,"Entrez l'adresse IP du serveur FTP."                   },
{ TXT_INPUT_DIR,      "Entrez le nom du rpertoire sur le serveur."           },
{ TXT_ENTER_PARAMS,   "Entrez les ventuels paramtres pour \"%s\"."          },
{ TXT_TRY_NFS_MOUNT,  "Tentative de monter le rpertoire\n\n\"%s:%s\"\n\n... "},
{ TXT_ERROR_NFSMOUNT, "Lors de la tentative de monter le rpertoire NFS du"
                      "serveur, il s'est produit l'erreur suivante:\n\n"
                      "Erreur  partir du serveur NFS: %s"                    },
{ TXT_CHOOSE_KEYMAP,  "Slectionnez une disposition du clavier.\nYaST "
                      "vous offrira plus tard d'autres dispositions."         },
{ TXT_DELETE_MODULE,  "Slectionnez le module que vous voulez supprimer."     },
{ TXT_NO_MODULES,     "Actuellement aucun module n'est charg!"               },
/* 'Recherche automatique' is correct, but is too long :-( */
{ TXT_AUTO_RUNNING,   "Autoprobing (%s)...      "                             },
{ TXT_WANT_MORE_SCSI, "Voulez-vous rechercher d'autres adaptateurs SCSI?"     },
{ TXT_WANT_MORE_NET,  "Voulez-vous rechercher d'autres cartes rseau?"        },
{ TXT_WANT_MORE_CDROM,"Voulez-vous rechercher d'autres lecteurs de CD-ROM?"   },
{ TXT_NET_ETH0,       "Carte rseau Ethernet"                                 },
{ TXT_NET_PLIP,       "Port parallle"                                        },
{ TXT_NET_TR0,        "Carte rseau Token Ring"                               },
{ TXT_NET_ARC0,       "Carte rseau Arcnet"                                   },
{ TXT_CHOOSE_NET,     "Slectionnez le type de rseau que vous voulez "
                      "utiliser pour l'installation."                         },
{ TXT_INPUT_PLIP_IP,  "Entrez l'adresse IP du partenaire PLIP."               },
{ TXT_LOAD_PCMCIA,    "Chargement des modules PCMCIA"                         },
{ TXT_NO_PCMCIA,      "Il n'a pas t dtect de matriel PCMCIA!"            },
{ TXT_FOUND_PCMCIA,   "Chipset PCMCIA \"%s\" dtect!\n\nEntrez les "
                      "ventuels paramtres pour les modules core PCMCIA "    },
{ TXT_PCMCIA_PARAMS,  "Entrez les ventuels paramtres spcifiques pour le "
                      "Chipset \"%s\""                                        },
{ TXT_START_CARDMGR,  "Lancement du gestionnaire de carte..."                 },
{ TXT_PCMCIA_SUCCESS, "Services de carte PCMCIA lancs avec succs."          },
{ TXT_PCMCIA_FAIL,    "Le lancement des services carte PCMCIA a chou!"      },
{ TXT_PCMCIA_ALREADY, "Les services carte PCMCIA tournent dj!"              },
{ TXT_NO_LXRC_END,    "Seule l'option du menu\n\n"                 
                      " Dmarrer l'installation / le systme \n\n"
                      "permet de quitter Linuxrc. "
                      "Pour que ce soit possible vous devez ventuellement "
                      "charger auparavant le pilote matriel (modules)."      },
{ TXT_NO_CTRLC,       "Vous ne pouvez pas quitter Linuxrc  l'aide de "
                      "<ctrl-c>!"                                             },
{ TXT_REMOVE_DISK,    "Retirez maintenant la disquette du lecteur!"           },
{ TXT_EXAMPLE_PAR,    "\n\nexemple: "                                         },
{ TXT_CHOOSE_SOURCE,  "Slectionnez le support source"                        },
{ TXT_TRY_CD_MOUNT,   "Tentative de monter le CD-ROM..."                      },
{ TXT_ERROR_CD_MOUNT, "Le CD-ROM n'a pas pu tre mont!"                      },
{ TXT_START_YAST,     "Dmarrage de YaST..."                                  },
{ TXT_ASK_REBOOT,     "Souhaitez-vous maintenant redmarrer le systme?"      },
{ TXT_INSTALL_SUCCESS,"Le systme de base a t install avec succs. Ce "
                      "systme est maintenant dmarr pour "
                      "terminer l'installation."                              },
{ TXT_ENTER_PARTITION,"Indiquez la partition du disque dur\n"
                      "(par exemple /dev/sda1)"                               },
{ TXT_ERROR_HD_MOUNT, "La partition indique n'a pas pu tre monte!"         },
{ TXT_ENTER_HD_DIR,   "Entrez le nom du rpertoire source"                    },
{ TXT_ERROR_INSTALL,  "L'installation n'a pas t mene  bonne fin!"         },
{ TXT_DRIVES,         "Disques durs / CD-ROMs"                                },
{ TXT_LITTLE_MEM,     "Votre machine ne dispose que de trs peu de mmoire "
                      "centrale!\n"
                      "\nAvant de confirmer ce message et avant que YaST "
                      "puisse tre lanc, vous devez passer  la console 2 "
                      "(appuyez sur ALT-F2) puis configurer et activer"
                      "manuellement la mmoire de swap!"                      },
{ TXT_ASK_ANIMATE,    "Animation"                                             },
{ TXT_ASK_EXPLODE,    "Voulez-vous des fentres \"explosives\"?\n\n"
                      "Bien sr ;-)"                                          },
{ TXT_NO_DRIVES,      "Jusqu' prsent il n'a t reconnu ni disques durs "
                      "ni lecteurs de CD-ROM."                                },
{ TXT_MENU_EXPERT,    "Dbogage (experts)"                                    },
{ TXT_FORCE_ROOTIMAGE,"Forage de l'image de root"                            },
{ TXT_NEW_ROOTIMAGE,  "Entre de l'image de root"                             },
{ TXT_ASK_RI_FORCE,   "Faut-il charger l'image de root dans le RAM-disque "
                      "mme s'il n'a pas t dtect de systme "
                      "d'installation valide?"                                },
{ TXT_ENTER_ROOTIMAGE,"Entrez le chemin et le nom du fichier que vous voulez "
                      "charger comme systme de fichiers racine dans le "        
                      "RAM-disque."                                           },
{ TXT_NOT_IMPLEMENTED,"Pas encore implment!"                                },
{ TXT_NEW_INST_SYS,   "Systme d'installation"                                },
{ TXT_ENTER_INST_SYS, "Entrez le nom du rpertoire contenant le "
                      "systme d'installation."                               },
{ TXT_NO_INFO_AVAIL,  "Aucune information disponible."                        },
{ TXT_PREPARE_INST,   "Prparation de l'installation..."                      },
{ TXT_START_DEMO,     "Dmarrer le CD live"                                   },
{ TXT_INSERT_LIVECD,  "Insrez le CD live."                                   },
{ TXT_END_REBOOT,     "Abandon / Ramorage"                                  },
{ TXT_ASK_BOOTP,      "Configuration automatique via Bootp?"                  },
{ TXT_SEND_BOOTP,     "La demande Bootp est envoye..."                       },
{ TXT_ERROR_BOOTP,    "La configuration Bootp a chou."                      },
{ TXT_SEARCH_INFOFILE,"Recherche du fichier info..."                          },
{ TXT_NFSPORT,        "Indication du port NFS"                                },
{ TXT_ASKING_NAMED,   "Interrogation du serveur de noms..."                   },
{ TXT_HOST_NOT_FOUND, "La machine %s est inconnue!"                           },
{ TXT_NET_CONFIGURED, "Votre rseau est dj configur. Voulez-vous "
                      "conserver la configuration?"                           },
{ TXT_ASK_BREAK,      "Voulez-vous abandonner?"                               },
{ TXT_NO_NETDEVICE,   "Il n'a pas t dtect de carte rseau!\nChargez un "
                      "module adquat."                             },
{ TXT_ALREADY_FOUND,  "Dans votre systme il a dj t dtect un %s!\n"     },
{ TXT_SCSI_ADAPTER,   "Adaptateur SCSI"                                       },
{ TXT_ANOTHER_MOD,    "Voulez-vous charger un autre module?"                  },
{ TXT_ERROR_FTP,      "Erreur lors de l'accs au serveur FTP:\n\n%s"          },
{ TXT_TRY_REACH_FTP,  "Tentative de joindre le serveur FTP..."                },
{ TXT_ENTER_FTPUSER,  "Entrez le nom d'utilisateur sous lequel vous "
                      "voulez accder au serveur FTP"                         },
{ TXT_ENTER_FTPPASSWD,"Entrez le mot de passe pour le compte utilisateur FTP" },
{ TXT_ENTER_FTPPROXY, "Indiquez l'adresse du proxy FTP"                       },
{ TXT_ENTER_FTPPORT,  "Indiquez le port du proxy FTP"                         },
{ TXT_WANT_FTPPROXY,  "Voulez-vous utiliser un proxy FTP?"                    },
{ TXT_NOMEM_FTP,      "Votre machine n'a pas suffisamment de RAM pour "
                      "une installatioin via FTP! (16Mo de RAM sont requis)"  },
{ TXT_ANONYM_FTP,     "Normalmente un'installazione FTP viene eseguita via "
                      "anonymus FTP; c`e` pero` la possibilita` di "
                      "soecificare un username e un password sul server FTP.\n"
                      "Vuole farlo?"                                          },
{ TXT_ENTER_MODDISK,  "Insrez la disquette modules"                          },
{ TXT_MORE_MODULES,   "-- Davantage de modules --"                            },
{ TXT_INIT_PARPORT,   "Initialisation du port parallle..."                   },
{ TXT_ENTER_NFSPORT,  "Entrez le numro du port sur lequel tourne le "
                      "serveur NFS"                                           }
};

char *txt_get (enum textid_t id_iv)
    {
    text_t  *txt_pci;
    int      found_ii = FALSE;
    int      i_ii = 0;


    if (language_ig == LANG_GERMAN)
        txt_pci = txt_german_atm;
    else if (language_ig == LANG_ITALIAN)
        txt_pci = txt_italian_atm;
    else if (language_ig == LANG_FRENCH)
        txt_pci = txt_french_atm;
    else
        txt_pci = txt_english_atm;

    while (!found_ii &&
           i_ii < sizeof (txt_german_atm) / sizeof (txt_german_atm [0]))
        if (txt_pci [i_ii].id == id_iv)
            found_ii = TRUE;
        else
            i_ii++;

    if (found_ii)
        return (txt_pci [i_ii].text);
    else
        return ("*** Unknown text! ***");
    }


int txt_init (void)
    {
    int  nr_texts_ii;

    nr_texts_ii = sizeof (txt_german_atm) / sizeof (txt_german_atm [0]);

    if (nr_texts_ii != sizeof (txt_english_atm) / sizeof (txt_english_atm [0]))
        {
        fprintf (stderr, "German: %d, English: %d\n", nr_texts_ii,
                 sizeof (txt_english_atm) / sizeof (txt_english_atm [0]));
        return (-1);
        }

    if (nr_texts_ii != sizeof (txt_italian_atm) / sizeof (txt_italian_atm [0]))
        {
        fprintf (stderr, "German: %d, Italian: %d\n", nr_texts_ii,
                 sizeof (txt_italian_atm) / sizeof (txt_italian_atm [0]));
        return (-1);
        }

    if (nr_texts_ii != sizeof (txt_french_atm) / sizeof (txt_french_atm [0]))
        {
        fprintf (stderr, "German: %d, French: %d\n", nr_texts_ii,
                 sizeof (txt_french_atm) / sizeof (txt_french_atm [0]));
        return (-1);
        }

    return (0);
    }
